@@ -43,6 +43,8 @@
 
 #ifdef SDL2
 #include <SDL2/SDL.h>
+#include <SDL2/SDL_video.h>
+
 #else // SDL1.2
 #include <SDL/SDL.h>
 #endif //SDL2
@@ -66,6 +68,7 @@ SDL_Surface* window = NULL;
 #endif
 
 qboolean have_stencil = false;
+qboolean vsync_active;
 
 #ifdef X11GAMMA
 XRRCrtcGamma** gammaRamps = NULL;
@@ -182,7 +185,7 @@ SetSDLIcon()
 	transColor.b = 255;
 
 	solidColor.r = 0;
-	solidColor.g = 16;
+	solidColor.g = 0;
 	solidColor.b = 0;
 
 	SDL_SetColors(icon, &transColor, 0, 1);
@@ -442,7 +445,10 @@ static void InitGamma()
 	XRRFreeScreenResources(res);
 
 	VID_Printf(PRINT_ALL, "Using hardware gamma via X11/xRandR.\n");
-
+#elif __APPLE__
+	gl_state.hwgamma = false;
+	VID_Printf(PRINT_ALL, "Using software gamma (needs vid_restart after changes)\n");
+	return;
 #else
 	VID_Printf(PRINT_ALL, "Using hardware gamma via SDL.\n");
 #endif
@@ -599,6 +605,7 @@ GLimp_InitGraphics(qboolean fullscreen)
 
 	/* Set vsync */
 	SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, gl_swapinterval->value ? 1 : 0);
+	vsync_active = gl_swapinterval->value ? true : false;
 #endif
 
 	while (1)
@@ -658,6 +665,7 @@ GLimp_InitGraphics(qboolean fullscreen)
 
 	/* Set vsync - TODO: -1 could be set for "late swap tearing" */
 	SDL_GL_SetSwapInterval(gl_swapinterval->value ? 1 : 0);
+	vsync_active = gl_swapinterval->value ? true : false;
 #endif
 
 	/* Initialize the stencil buffer */
@@ -747,6 +755,37 @@ void GLimp_GrabInput(qboolean grab)
 #endif
 }
 
+/*
+ * Returns the VSync state.
+ */
+qboolean GLimp_VsyncEnabled(void)
+{
+	return vsync_active;
+}
+
+/*
+ * Returns the current display refresh rate.
+ */
+int GLimp_GetRefreshRate(void)
+{
+#if SDL_VERSION_ATLEAST(2, 0, 0)
+	int i;
+	int refresh = 0;
+	SDL_DisplayMode mode;
+
+	for (i = 0; i < SDL_GetNumVideoDisplays(); ++i)
+	{
+		SDL_GetCurrentDisplayMode(i, &mode);
+		refresh = refresh < mode.refresh_rate ? mode.refresh_rate : refresh;
+	}
+
+	return refresh;
+#else
+	// Asume 60hz.
+	return 60
+#endif
+}
+
 /*
  * Shuts the SDL render backend down
  */
