@@ -37,21 +37,10 @@ void CL_CheckForResend(void);
 
 cvar_t *freelook;
 
-cvar_t *adr0;
-cvar_t *adr1;
-cvar_t *adr2;
-cvar_t *adr3;
-cvar_t *adr4;
-cvar_t *adr5;
-cvar_t *adr6;
-cvar_t *adr7;
-cvar_t *adr8;
-
 cvar_t *rcon_client_password;
 cvar_t *rcon_address;
 
 cvar_t *cl_noskins;
-cvar_t *cl_autoskins;
 cvar_t *cl_footsteps;
 cvar_t *cl_timeout;
 cvar_t *cl_predict;
@@ -62,6 +51,7 @@ cvar_t *cl_add_particles;
 cvar_t *cl_add_lights;
 cvar_t *cl_add_entities;
 cvar_t *cl_add_blend;
+cvar_t *cl_async;
 
 cvar_t *cl_shownet;
 cvar_t *cl_showmiss;
@@ -82,8 +72,6 @@ cvar_t *m_side;
 cvar_t *cl_lightlevel;
 
 /* userinfo */
-cvar_t *info_password;
-cvar_t *info_spectator;
 cvar_t *name;
 cvar_t *skin;
 cvar_t *rate;
@@ -95,6 +83,12 @@ cvar_t *hand;
 cvar_t *gender;
 cvar_t *gender_auto;
 
+cvar_t *gl_maxfps;
+
+cvar_t	*gl_stereo;
+cvar_t	*gl_stereo_separation;
+cvar_t	*gl_stereo_convergence;
+
 cvar_t *cl_vwep;
 
 client_static_t cls;
@@ -479,19 +473,9 @@ CL_InitLocal(void)
 
 	CL_InitInput();
 
-	adr0 = Cvar_Get("adr0", "", CVAR_ARCHIVE);
-	adr1 = Cvar_Get("adr1", "", CVAR_ARCHIVE);
-	adr2 = Cvar_Get("adr2", "", CVAR_ARCHIVE);
-	adr3 = Cvar_Get("adr3", "", CVAR_ARCHIVE);
-	adr4 = Cvar_Get("adr4", "", CVAR_ARCHIVE);
-	adr5 = Cvar_Get("adr5", "", CVAR_ARCHIVE);
-	adr6 = Cvar_Get("adr6", "", CVAR_ARCHIVE);
-	adr7 = Cvar_Get("adr7", "", CVAR_ARCHIVE);
-	adr8 = Cvar_Get("adr8", "", CVAR_ARCHIVE);
-
+	/* register our variables */
 	cin_force43 = Cvar_Get("cin_force43", "1", 0);
 
-	/* register our variables */
 	cl_add_blend = Cvar_Get("cl_blend", "1", 0);
 	cl_add_lights = Cvar_Get("cl_lights", "1", 0);
 	cl_add_particles = Cvar_Get("cl_particles", "1", 0);
@@ -499,10 +483,10 @@ CL_InitLocal(void)
 	cl_gun = Cvar_Get("cl_gun", "2", CVAR_ARCHIVE);
 	cl_footsteps = Cvar_Get("cl_footsteps", "1", 0);
 	cl_noskins = Cvar_Get("cl_noskins", "0", 0);
-	cl_autoskins = Cvar_Get("cl_autoskins", "0", 0);
 	cl_predict = Cvar_Get("cl_predict", "1", 0);
-	cl_maxfps = Cvar_Get("cl_maxfps", "95", CVAR_ARCHIVE);
+	cl_maxfps = Cvar_Get("cl_maxfps", "60", CVAR_ARCHIVE);
 	cl_drawfps = Cvar_Get("cl_drawfps", "0", CVAR_ARCHIVE);
+	cl_async = Cvar_Get("cl_async", "1", CVAR_ARCHIVE);
 
 	cl_upspeed = Cvar_Get("cl_upspeed", "200", 0);
 	cl_forwardspeed = Cvar_Get("cl_forwardspeed", "200", 0);
@@ -529,14 +513,18 @@ CL_InitLocal(void)
 	cl_paused = Cvar_Get("paused", "0", 0);
 	cl_timedemo = Cvar_Get("timedemo", "0", 0);
 
+	gl_maxfps = Cvar_Get("gl_maxfps", "95", CVAR_ARCHIVE);
+
+	gl_stereo = Cvar_Get( "gl_stereo", "0", CVAR_ARCHIVE );
+	gl_stereo_separation = Cvar_Get( "gl_stereo_separation", "1", CVAR_ARCHIVE );
+	gl_stereo_convergence = Cvar_Get( "gl_stereo_convergence", "1.4", CVAR_ARCHIVE );
+
 	rcon_client_password = Cvar_Get("rcon_password", "", 0);
 	rcon_address = Cvar_Get("rcon_address", "", 0);
 
 	cl_lightlevel = Cvar_Get("gl_lightlevel", "0", 0);
 
 	/* userinfo */
-	info_password = Cvar_Get("password", "", CVAR_USERINFO);
-	info_spectator = Cvar_Get("spectator", "0", CVAR_USERINFO);
 	name = Cvar_Get("name", "unnamed", CVAR_USERINFO | CVAR_ARCHIVE);
 	skin = Cvar_Get("skin", "male/grunt", CVAR_USERINFO | CVAR_ARCHIVE);
 	rate = Cvar_Get("rate", "8000", CVAR_USERINFO | CVAR_ARCHIVE);
@@ -630,6 +618,8 @@ CL_WriteConfiguration(void)
 	fprintf(f, "// generated by quake, do not modify\n");
 
 	Key_WriteBindings(f);
+
+	fflush(f);
 	fclose(f);
 
 	Cvar_WriteVariables(path);
@@ -717,66 +707,68 @@ CL_UpdateWindowedMouse(void)
 	}
 }
 
-void
-CL_SendCommand(void)
-{
-	/* update windowed_mouse cvar */
-	CL_UpdateWindowedMouse();
-
-	/* get new key events */
-	Sys_SendKeyEvents();
-
-	/* process console commands */
-	Cbuf_Execute();
-
-	/* fix any cheating cvars */
-	CL_FixCvarCheats();
-
-	/* send intentions now */
-	CL_SendCmd();
-
-	/* resend a connection request if necessary */
-	CL_CheckForResend();
-}
+qboolean GLimp_VsyncEnabled(void);
+int GLimp_GetRefreshRate(void);
 
 void
 CL_Frame(int msec)
 {
-	static int extratime;
+	int nfps;
+	int rfps;
+
 	static int lasttimecalled;
 
+	static int packetdelta = 1000;
+	static int renderdelta = 1000;
+	static int miscdelta = 1000;
+
+	qboolean packetframe = true;
+	qboolean renderframe = true;
+	qboolean miscframe = true;
+
 	if (dedicated->value)
 	{
 		return;
 	}
 
-	extratime += msec;
-
-	if (!cl_timedemo->value)
+	// Target render frame rate
+	if (GLimp_VsyncEnabled())
 	{
-		if ((cls.state == ca_connected) && (extratime < 100))
-		{
-			return; /* don't flood packets out while connecting */
-		}
+		rfps = GLimp_GetRefreshRate();
 
-		if (extratime < 1000 / cl_maxfps->value)
+		if (rfps > gl_maxfps->value)
 		{
-			return; /* framerate is too high */
+			rfps = (int)gl_maxfps->value;
 		}
 	}
-
-	/* decide the simulation time */
-	cls.frametime = extratime / 1000.0;
-
-	cl.time += extratime;
-
-	cls.realtime = curtime;
-
-	extratime = 0;
-
-	if (cls.frametime > (1.0 / 5))
+	else
 	{
-		cls.frametime = (1.0 / 5);
+		rfps = (int)gl_maxfps->value;
+	}
+
+	// The network framerate must not be higher then the render framerate
+	nfps = (cl_maxfps->value > rfps) ? rfps : cl_maxfps->value;
+
+	// Adjust deltas
+	packetdelta += msec;
+	renderdelta += msec;
+	miscdelta += msec;
+
+	// Calculate simulation time
+	cls.nframetime = packetdelta * 0.001f;
+	cls.rframetime = renderdelta * 0.001f;
+	cls.realtime = curtime;
+	cl.time += msec;
+
+	// Don't extrapolate too far ahead
+	if (cls.nframetime > 0.5f)
+	{
+		cls.nframetime = 0.5f;
+	}
+
+	if (cls.rframetime > 0.5f)
+	{
+		cls.rframetime = 0.5f;
 	}
 
 	/* if in the debugger last frame, don't timeout */
@@ -785,77 +777,176 @@ CL_Frame(int msec)
 		cls.netchan.last_received = Sys_Milliseconds();
 	}
 
-	/* fetch results from server */
-	CL_ReadPackets();
-
-	/* send a new command message to the server */
-	CL_SendCommand();
-
-	/* predict all unacknowledged movements */
-	CL_PredictMovement();
-
-	/* allow renderer DLL change */
-	VID_CheckChanges();
-
-	if (!cl.refresh_prepped && (cls.state == ca_active))
+	if (!cl_timedemo->value)
 	{
-		CL_PrepRefresh();
-	}
-
-	/* update the screen */
-	if (host_speeds->value)
-	{
-		time_before_ref = Sys_Milliseconds();
-	}
-
-	SCR_UpdateScreen();
-
-	if (host_speeds->value)
-	{
-		time_after_ref = Sys_Milliseconds();
-	}
-
-	/* update audio */
-	S_Update(cl.refdef.vieworg, cl.v_forward, cl.v_right, cl.v_up);
-#ifdef CDA
-	CDAudio_Update();
-#endif
-
-	/* advance local effects for next frame */
-	CL_RunDLights();
-
-	CL_RunLightStyles();
-
-	SCR_RunCinematic();
-
-	SCR_RunConsole();
-
-	cls.framecount++;
-
-	if (log_stats->value)
-	{
-		if (cls.state == ca_active)
+		// Don't flood while connecting
+		if ((cls.state == ca_connected) && (packetdelta < 100))
 		{
-			if (!lasttimecalled)
-			{
-				lasttimecalled = Sys_Milliseconds();
+			packetframe = false;
+		}
 
-				if (log_stats_file)
-				{
-					fprintf(log_stats_file, "0\n");
-				}
+		if (cl_async->value)
+		{
+			// Network frames
+			if (packetdelta < (1000.0f / nfps))
+			{
+				packetframe = false;
+			}
+			else if (cls.nframetime == cls.rframetime)
+			{
+				packetframe = false;
 			}
 
-			else
+			// Render frames
+			if (renderdelta < (1000.0f / rfps))
 			{
-				int now = Sys_Milliseconds();
+				renderframe = false;
+			}
 
-				if (log_stats_file)
+			// Misc. stuff at 10 FPS
+			if (miscdelta < 100.0f)
+			{
+				miscframe = false;
+			}
+		}
+		else
+		{
+			// Cap frames at gl_maxfps
+			if (renderdelta < (1000.0f / rfps))
+			{
+				renderframe = false;
+				packetframe = false;
+				miscframe = false;
+			}
+		}
+
+		// Throttle the game a little bit. 1000 FPS are enough.
+		if (!packetframe && !renderframe && !cls.forcePacket && !userinfo_modified)
+		{
+			double frametime = (1000.0 / cl_maxfps->value - packetdelta) <= (1000.0 / gl_maxfps->value - renderdelta) ?
+							   (1000.0 / cl_maxfps->value - packetdelta) : (1000.0 / gl_maxfps->value - renderdelta);
+
+			if (frametime > 1)
+			{
+				Sys_Sleep(1);
+			}
+
+			return;
+		}
+	}
+	else if (msec < 1)
+	{
+		return;
+	}
+
+	// Update input stuff
+	if (packetframe || renderframe)
+	{
+		CL_ReadPackets();
+		CL_UpdateWindowedMouse();
+		Sys_SendKeyEvents();
+		Cbuf_Execute();
+		CL_FixCvarCheats();
+
+		if (cls.state > ca_connecting)
+		{
+			CL_RefreshCmd();
+		}
+		else
+		{
+			CL_RefreshMove();
+		}
+	}
+
+	if (cls.forcePacket || userinfo_modified)
+	{
+		packetframe = true;
+		cls.forcePacket = false;
+	}
+
+	if (packetframe)
+	{
+		packetdelta = 0;
+
+		CL_SendCmd();
+		CL_CheckForResend();
+	}
+
+	if (renderframe)
+	{
+		renderdelta = 0;
+
+		if (miscframe)
+		{
+			miscdelta = 0;
+
+			VID_CheckChanges();
+		}
+
+		CL_PredictMovement();
+
+		if (!cl.refresh_prepped && (cls.state == ca_active))
+		{
+			CL_PrepRefresh();
+		}
+
+		/* update the screen */
+		if (host_speeds->value)
+		{
+			time_before_ref = Sys_Milliseconds();
+		}
+
+		SCR_UpdateScreen();
+
+		if (host_speeds->value)
+		{
+			time_after_ref = Sys_Milliseconds();
+		}
+
+		/* update audio */
+		S_Update(cl.refdef.vieworg, cl.v_forward, cl.v_right, cl.v_up);
+
+#ifdef CDA
+		if (miscframe)
+		{
+			CDAudio_Update();
+		}
+#endif
+
+		/* advance local effects for next frame */
+		CL_RunDLights();
+		CL_RunLightStyles();
+		SCR_RunCinematic();
+		SCR_RunConsole();
+
+		/* Update framecounter */
+		cls.framecount++;
+
+		if (log_stats->value)
+		{
+			if (cls.state == ca_active)
+			{
+				if (!lasttimecalled)
 				{
-					fprintf(log_stats_file, "%d\n", now - lasttimecalled);
+					lasttimecalled = Sys_Milliseconds();
+
+					if (log_stats_file)
+					{
+						fprintf(log_stats_file, "0\n");
+					}
 				}
 
-				lasttimecalled = now;
+				else
+				{
+					int now = Sys_Milliseconds();
+
+					if (log_stats_file)
+					{
+						fprintf(log_stats_file, "%d\n", now - lasttimecalled);
+					}
+
+					lasttimecalled = now;
+				}
 			}
 		}
 	}
