@@ -210,7 +210,8 @@ typedef struct
 
 	int			framecount;
 	int			realtime; /* always increasing, no clamping, etc */
-	float		frametime; /* seconds since last frame */
+	float		rframetime; /* seconds since last render frame */
+	float		nframetime; /* network frame time */
 
 	/* screen rendering information */
 	float		disable_screen; /* showing loading plaque between levels */
@@ -231,6 +232,8 @@ typedef struct
 
 	int			challenge; /* from the server to use for connecting */
 
+	qboolean	forcePacket; /* Forces a package to be send at the next frame. */
+
 	FILE		*download; /* file transfer from server */
 	char		downloadtempname[MAX_OSPATH];
 	char		downloadname[MAX_OSPATH];
@@ -258,7 +261,6 @@ extern	cvar_t	*cl_add_entities;
 extern	cvar_t	*cl_predict;
 extern	cvar_t	*cl_footsteps;
 extern	cvar_t	*cl_noskins;
-extern	cvar_t	*cl_autoskins;
 extern	cvar_t	*cl_upspeed;
 extern	cvar_t	*cl_forwardspeed;
 extern	cvar_t	*cl_sidespeed;
@@ -438,7 +440,9 @@ extern 	kbutton_t 	in_strafe;
 extern 	kbutton_t 	in_speed;
 
 void CL_InitInput (void);
+void CL_RefreshCmd(void);
 void CL_SendCmd (void);
+void CL_RefreshMove(void);
 void CL_SendMove (usercmd_t *cmd);
 
 void CL_ClearState (void);
