@@ -28,12 +28,13 @@
 #include "../../client/header/client.h"
 #include "../../client/menu/header/qmenu.h"
 
-#define CUSTOM_MODE 24
-
 extern void M_ForceMenuOff(void);
 
 static cvar_t *gl_mode;
 static cvar_t *gl_hudscale;
+static cvar_t *gl_consolescale;
+static cvar_t *gl_menuscale;
+static cvar_t *crosshair_scale;
 static cvar_t *fov;
 extern cvar_t *scr_viewsize;
 extern cvar_t *vid_gamma;
@@ -46,7 +47,7 @@ static menuframework_s s_opengl_menu;
 
 static menulist_s s_mode_list;
 static menulist_s s_aspect_list;
-static menulist_s s_hudscale_list;
+static menulist_s s_uiscale_list;
 static menuslider_s s_screensize_slider;
 static menuslider_s s_brightness_slider;
 static menulist_s s_fs_box;
@@ -56,6 +57,27 @@ static menulist_s s_msaa_list;
 static menuaction_s s_defaults_action;
 static menuaction_s s_apply_action;
 
+static int
+GetCustomValue(menulist_s *list)
+{
+	static menulist_s *last;
+	static int i;
+
+	if (list != last)
+	{
+		last = list;
+		i = list->curvalue;
+		do
+		{
+			i++;
+		}
+		while (list->itemnames[i]);
+		i--;
+	}
+
+	return i;
+}
+
 static void
 ScreenSizeCallback(void *s)
 {
@@ -100,7 +122,7 @@ ApplyChanges(void *unused)
 	qboolean restart = false;
 
 	/* custom mode */
-	if (s_mode_list.curvalue != CUSTOM_MODE)
+	if (s_mode_list.curvalue != GetCustomValue(&s_mode_list))
 	{
 		/* Restarts automatically */
 		Cvar_SetValue("gl_mode", s_mode_list.curvalue);
@@ -161,30 +183,21 @@ ApplyChanges(void *unused)
 		}
 	}
 
-	/* HUD scaling */
-	if (s_hudscale_list.curvalue == 0)
-	{
-		Cvar_SetValue("gl_hudscale", 1);
-	}
-	else if (s_hudscale_list.curvalue == 1)
+	/* UI scaling */
+	if (s_uiscale_list.curvalue == 0)
 	{
 		Cvar_SetValue("gl_hudscale", -1);
 	}
-	else if (s_hudscale_list.curvalue == 2)
+	else if (s_uiscale_list.curvalue < GetCustomValue(&s_uiscale_list))
 	{
-		Cvar_SetValue("gl_hudscale", 1.5);
+		Cvar_SetValue("gl_hudscale", s_uiscale_list.curvalue);
 	}
-	else if (s_hudscale_list.curvalue == 3)
+
+	if (s_uiscale_list.curvalue != GetCustomValue(&s_uiscale_list))
 	{
-		Cvar_SetValue("gl_hudscale", 2);
-	}
-	else if (s_hudscale_list.curvalue == 4)
-	{
-		Cvar_SetValue("gl_hudscale", 2.5);
-	}
-	else if (s_hudscale_list.curvalue == 5)
-	{
-		Cvar_SetValue("gl_hudscale", 3);
+		Cvar_SetValue("gl_consolescale", gl_hudscale->value);
+		Cvar_SetValue("gl_menuscale", gl_hudscale->value);
+		Cvar_SetValue("crosshair_scale", gl_hudscale->value);
 	}
 
 	/* Restarts automatically */
@@ -267,13 +280,15 @@ VID_MenuInit(void)
 		0
 	};
 
-	static const char *hudscale_names[] = {
-		"no (1x)",
+	static const char *uiscale_names[] = {
 		"auto",
-		"1.5x",
+		"1x",
 		"2x",
-		"2.5x",
-		"3.0x",
+		"3x",
+		"4x",
+		"5x",
+		"6x",
+		"custom",
 		0
 	};
 
@@ -301,6 +316,18 @@ VID_MenuInit(void)
 	{
 		gl_hudscale = Cvar_Get("gl_hudscale", "-1", CVAR_ARCHIVE);
 	}
+	if (!gl_consolescale)
+	{
+		gl_consolescale = Cvar_Get("gl_consolescale", "-1", CVAR_ARCHIVE);
+	}
+	if (!gl_menuscale)
+	{
+		gl_menuscale = Cvar_Get("gl_menuscale", "-1", CVAR_ARCHIVE);
+	}
+	if (!crosshair_scale)
+	{
+		crosshair_scale = Cvar_Get("crosshair_scale", "-1", CVAR_ARCHIVE);
+	}
 
 	if (!horplus)
 	{
@@ -351,7 +378,7 @@ VID_MenuInit(void)
 	}
 	else
 	{
-		s_mode_list.curvalue = CUSTOM_MODE;
+		s_mode_list.curvalue = GetCustomValue(&s_mode_list);
 	}
 
 	s_aspect_list.generic.type = MTYPE_SPINCONTROL;
@@ -381,37 +408,33 @@ VID_MenuInit(void)
 	}
 	else
 	{
-		s_aspect_list.curvalue = 5;
+		s_aspect_list.curvalue = GetCustomValue(&s_aspect_list);
 	}
 
-	s_hudscale_list.generic.type = MTYPE_SPINCONTROL;
-	s_hudscale_list.generic.name = "hud scale";
-	s_hudscale_list.generic.x = 0;
-	s_hudscale_list.generic.y = (y += 10);
-	s_hudscale_list.itemnames = hudscale_names;
-	if (gl_hudscale->value == 1)
+	s_uiscale_list.generic.type = MTYPE_SPINCONTROL;
+	s_uiscale_list.generic.name = "ui scale";
+	s_uiscale_list.generic.x = 0;
+	s_uiscale_list.generic.y = (y += 10);
+	s_uiscale_list.itemnames = uiscale_names;
+	if (gl_hudscale->value != gl_consolescale->value ||
+		gl_hudscale->value != gl_menuscale->value ||
+		gl_hudscale->value != crosshair_scale->value)
 	{
-		s_hudscale_list.curvalue = 0;
+		s_uiscale_list.curvalue = GetCustomValue(&s_uiscale_list);
 	}
 	else if (gl_hudscale->value < 0)
 	{
-		s_hudscale_list.curvalue = 1;
+		s_uiscale_list.curvalue = 0;
 	}
-	else if (gl_hudscale->value == 1.5f)
+	else if (gl_hudscale->value > 0 &&
+			gl_hudscale->value < GetCustomValue(&s_uiscale_list) &&
+			gl_hudscale->value == (int)gl_hudscale->value)
 	{
-		s_hudscale_list.curvalue = 2;
+		s_uiscale_list.curvalue = gl_hudscale->value;
 	}
-	else if (gl_hudscale->value == 2)
+	else
 	{
-		s_hudscale_list.curvalue = 3;
-	}
-	else if (gl_hudscale->value == 2.5f)
-	{
-		s_hudscale_list.curvalue = 4;
-	}
-	else if (gl_hudscale->value == 3)
-	{
-		s_hudscale_list.curvalue = 5;
+		s_uiscale_list.curvalue = GetCustomValue(&s_uiscale_list);
 	}
 
 	s_screensize_slider.generic.type = MTYPE_SLIDER;
@@ -493,7 +516,7 @@ VID_MenuInit(void)
 
 	Menu_AddItem(&s_opengl_menu, (void *)&s_mode_list);
 	Menu_AddItem(&s_opengl_menu, (void *)&s_aspect_list);
-	Menu_AddItem(&s_opengl_menu, (void *)&s_hudscale_list);
+	Menu_AddItem(&s_opengl_menu, (void *)&s_uiscale_list);
 	Menu_AddItem(&s_opengl_menu, (void *)&s_screensize_slider);
 	Menu_AddItem(&s_opengl_menu, (void *)&s_brightness_slider);
 	Menu_AddItem(&s_opengl_menu, (void *)&s_fs_box);
