@@ -90,16 +90,29 @@ Draw_CharScaled(int x, int y, int num, float scale)
 
 	R_Bind(draw_chars->texnum);
 
-	glBegin(GL_QUADS);
-	glTexCoord2f(fcol, frow);
-	glVertex2f(x, y);
-	glTexCoord2f(fcol + size, frow);
-	glVertex2f(x + scaledSize, y);
-	glTexCoord2f(fcol + size, frow + size);
-	glVertex2f(x + scaledSize, y + scaledSize);
-	glTexCoord2f(fcol, frow + size);
-	glVertex2f(x, y + scaledSize);
-	glEnd();
+	GLfloat vtx[] = {
+		x, y,
+		x + scaledSize, y,
+		x + scaledSize, y + scaledSize,
+		x, y + scaledSize
+	};
+
+	GLfloat tex[] = {
+		fcol, frow,
+		fcol + size, frow,
+		fcol + size, frow + size,
+		fcol, frow + size
+	};
+
+	glEnableClientState( GL_VERTEX_ARRAY );
+	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+
+	glVertexPointer( 2, GL_FLOAT, 0, vtx );
+	glTexCoordPointer( 2, GL_FLOAT, 0, tex );
+	glDrawArrays( GL_TRIANGLE_FAN, 0, 4 );
+
+	glDisableClientState( GL_VERTEX_ARRAY );
+	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
 }
 
 image_t *
@@ -157,16 +170,30 @@ Draw_StretchPic(int x, int y, int w, int h, char *pic)
 	}
 
 	R_Bind(gl->texnum);
-	glBegin(GL_QUADS);
-	glTexCoord2f(gl->sl, gl->tl);
-	glVertex2f(x, y);
-	glTexCoord2f(gl->sh, gl->tl);
-	glVertex2f(x + w, y);
-	glTexCoord2f(gl->sh, gl->th);
-	glVertex2f(x + w, y + h);
-	glTexCoord2f(gl->sl, gl->th);
-	glVertex2f(x, y + h);
-	glEnd();
+
+	GLfloat vtx[] = {
+		x, y,
+		x + w, y,
+		x + w, y + h,
+		x, y + h
+	};
+
+	GLfloat tex[] = {
+		gl->sl, gl->tl,
+		gl->sh, gl->tl,
+		gl->sh, gl->th,
+		gl->sl, gl->th
+	};
+
+	glEnableClientState( GL_VERTEX_ARRAY );
+	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+
+	glVertexPointer( 2, GL_FLOAT, 0, vtx );
+	glTexCoordPointer( 2, GL_FLOAT, 0, tex );
+	glDrawArrays( GL_TRIANGLE_FAN, 0, 4 );
+
+	glDisableClientState( GL_VERTEX_ARRAY );
+	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
 }
 
 void
@@ -193,20 +220,31 @@ Draw_PicScaled(int x, int y, char *pic, float factor)
 		Scrap_Upload();
 	}
 
-	GLfloat w = gl->width*factor;
-	GLfloat h = gl->height*factor;
-
 	R_Bind(gl->texnum);
-	glBegin(GL_QUADS);
-	glTexCoord2f(gl->sl, gl->tl);
-	glVertex2f(x, y);
-	glTexCoord2f(gl->sh, gl->tl);
-	glVertex2f(x + w, y);
-	glTexCoord2f(gl->sh, gl->th);
-	glVertex2f(x + w, y + h);
-	glTexCoord2f(gl->sl, gl->th);
-	glVertex2f(x, y + h);
-	glEnd();
+
+	GLfloat vtx[] = {
+		x, y,
+		x + gl->width * factor, y,
+		x + gl->width * factor, y + gl->height * factor,
+		x, y + gl->height * factor
+	};
+
+	GLfloat tex[] = {
+		gl->sl, gl->tl,
+		gl->sh, gl->tl,
+		gl->sh, gl->th,
+		gl->sl, gl->th
+	};
+
+	glEnableClientState( GL_VERTEX_ARRAY );
+	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+
+	glVertexPointer( 2, GL_FLOAT, 0, vtx );
+	glTexCoordPointer( 2, GL_FLOAT, 0, tex );
+	glDrawArrays( GL_TRIANGLE_FAN, 0, 4 );
+
+	glDisableClientState( GL_VERTEX_ARRAY );
+	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
 }
 
 /*
@@ -228,16 +266,30 @@ Draw_TileClear(int x, int y, int w, int h, char *pic)
 	}
 
 	R_Bind(image->texnum);
-	glBegin(GL_QUADS);
-	glTexCoord2f(x / 64.0, y / 64.0);
-	glVertex2f(x, y);
-	glTexCoord2f((x + w) / 64.0, y / 64.0);
-	glVertex2f(x + w, y);
-	glTexCoord2f((x + w) / 64.0, (y + h) / 64.0);
-	glVertex2f(x + w, y + h);
-	glTexCoord2f(x / 64.0, (y + h) / 64.0);
-	glVertex2f(x, y + h);
-	glEnd();
+
+	GLfloat vtx[] = {
+		x, y,
+		x + w, y,
+		x + w, y + h,
+		x, y + h
+	};
+
+	GLfloat tex[] = {
+		x / 64.0, y / 64.0,
+		( x + w ) / 64.0, y / 64.0,
+		( x + w ) / 64.0, ( y + h ) / 64.0,
+		x / 64.0, ( y + h ) / 64.0
+	};
+
+	glEnableClientState( GL_VERTEX_ARRAY );
+	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+
+	glVertexPointer( 2, GL_FLOAT, 0, vtx );
+	glTexCoordPointer( 2, GL_FLOAT, 0, tex );
+	glDrawArrays( GL_TRIANGLE_FAN, 0, 4 );
+
+	glDisableClientState( GL_VERTEX_ARRAY );
+	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
 }
 
 /*
@@ -260,18 +312,24 @@ Draw_Fill(int x, int y, int w, int h, int c)
 	glDisable(GL_TEXTURE_2D);
 
 	color.c = d_8to24table[c];
-	glColor3f(color.v[0] / 255.0, color.v[1] / 255.0,
-			color.v[2] / 255.0);
+	glColor4f(color.v [ 0 ] / 255.0, color.v [ 1 ] / 255.0,
+			   color.v [ 2 ] / 255.0, 1);
 
-	glBegin(GL_QUADS);
+	GLfloat vtx[] = {
+		x, y,
+		x + w, y,
+		x + w, y + h,
+		x, y + h
+	};
 
-	glVertex2f(x, y);
-	glVertex2f(x + w, y);
-	glVertex2f(x + w, y + h);
-	glVertex2f(x, y + h);
+	glEnableClientState( GL_VERTEX_ARRAY );
 
-	glEnd();
-	glColor3f(1, 1, 1);
+	glVertexPointer( 2, GL_FLOAT, 0, vtx );
+	glDrawArrays( GL_TRIANGLE_FAN, 0, 4 );
+
+	glDisableClientState( GL_VERTEX_ARRAY );
+
+	glColor4f( 1, 1, 1, 1 );
 	glEnable(GL_TEXTURE_2D);
 }
 
@@ -281,14 +339,21 @@ Draw_FadeScreen(void)
 	glEnable(GL_BLEND);
 	glDisable(GL_TEXTURE_2D);
 	glColor4f(0, 0, 0, 0.8);
-	glBegin(GL_QUADS);
 
-	glVertex2f(0, 0);
-	glVertex2f(vid.width, 0);
-	glVertex2f(vid.width, vid.height);
-	glVertex2f(0, vid.height);
+	GLfloat vtx[] = {
+		0, 0,
+		vid.width, 0,
+		vid.width, vid.height,
+		0, vid.height
+	};
+
+	glEnableClientState( GL_VERTEX_ARRAY );
+
+	glVertexPointer( 2, GL_FLOAT, 0, vtx );
+	glDrawArrays( GL_TRIANGLE_FAN, 0, 4 );
+
+	glDisableClientState( GL_VERTEX_ARRAY );
 
-	glEnd();
 	glColor4f(1, 1, 1, 1);
 	glEnable(GL_TEXTURE_2D);
 	glDisable(GL_BLEND);
@@ -297,61 +362,136 @@ Draw_FadeScreen(void)
 void
 Draw_StretchRaw(int x, int y, int w, int h, int cols, int rows, byte *data)
 {
-	unsigned image32[256 * 256];
-	unsigned char image8[256 * 256];
-	int i, j, trows;
+	GLfloat tex[8];
 	byte *source;
-	int frac, fracstep;
 	float hscale;
+	int frac, fracstep;
+	int i, j, trows;
 	int row;
-	float t;
 
 	R_Bind(0);
 
-	if (rows <= 256)
+	if(gl_config.npottextures || rows <= 256)
 	{
-		hscale = 1;
-		trows = rows;
+		// X, X
+		tex[0] = 0;
+		tex[1] = 0;
+
+		// X, Y
+		tex[2] = 1;
+		tex[3] = 0;
+
+		// Y, X
+		tex[4] = 1;
+		tex[5] = 1;
+
+		// Y, Y
+		tex[6] = 0;
+		tex[7] = 1;
 	}
 	else
 	{
+		// Scale params
 		hscale = rows / 256.0;
 		trows = 256;
+
+		// X, X
+		tex[0] = 1.0 / 512.0;
+		tex[1] = 1.0 / 512.0;
+
+		// X, Y
+		tex[2] = 511.0 / 512.0;
+		tex[3] = 1.0 / 512.0;
+
+		// Y, X
+		tex[4] = 511.0 / 512.0;
+		tex[5] = rows * hscale / 256 - 1.0 / 512.0;
+
+		// Y, Y
+		tex[6] = 1.0 / 512.0;
+		tex[7] = rows * hscale / 256 - 1.0 / 512.0;
 	}
 
-	t = rows * hscale / 256 - 1.0 / 512.0;
+	GLfloat vtx[] = {
+			x, y,
+			x + w, y,
+			x + w, y + h,
+			x, y + h
+	};
 
-	if (!qglColorTableEXT)
+	if (!gl_config.palettedtexture)
 	{
-		unsigned *dest;
+		unsigned image32[320*240]; /* was 256 * 256, but we want a bit more space */
 
-		for (i = 0; i < trows; i++)
+		/* .. because now if non-power-of-2 textures are supported, we just load
+		 * the data into a texture in the original format, without skipping any
+		 * pixels to fit into a 256x256 texture.
+		 * This causes text in videos (which are 320x240) to not look broken anymore.
+		 */
+		if(gl_config.npottextures || rows <= 256)
 		{
-			row = (int)(i * hscale);
+			unsigned* img = image32;
 
-			if (row > rows)
+			if(cols*rows > 320*240)
 			{
-				break;
+				/* in case there is a bigger video after all,
+				 * malloc enough space to hold the frame */
+				img = (unsigned*)malloc(cols*rows*4);
 			}
 
-			source = data + cols * row;
-			dest = &image32[i * 256];
-			fracstep = cols * 0x10000 / 256;
-			frac = fracstep >> 1;
-
-			for (j = 0; j < 256; j++)
+			for(i=0; i<rows; ++i)
 			{
-				dest[j] = r_rawpalette[source[frac >> 16]];
-				frac += fracstep;
+				int rowOffset = i*cols;
+				for(j=0; j<cols; ++j)
+				{
+					byte palIdx = data[rowOffset+j];
+					img[rowOffset+j] = r_rawpalette[palIdx];
+				}
+			}
+
+			glTexImage2D(GL_TEXTURE_2D, 0, gl_tex_solid_format,
+								cols, rows, 0, GL_RGBA, GL_UNSIGNED_BYTE,
+								img);
+
+			if(img != image32)
+			{
+				free(img);
 			}
 		}
+		else
+		{
+			unsigned int image32[320*240];
+			unsigned *dest;
 
-		glTexImage2D(GL_TEXTURE_2D, 0, gl_tex_solid_format,
-				256, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE,
-				image32);
+			for (i = 0; i < trows; i++)
+			{
+				row = (int)(i * hscale);
+
+				if (row > rows)
+				{
+					break;
+				}
+
+				source = data + cols * row;
+				dest = &image32[i * 256];
+				fracstep = cols * 0x10000 / 256;
+				frac = fracstep >> 1;
+
+				for (j = 0; j < 256; j++)
+				{
+					dest[j] = r_rawpalette[source[frac >> 16]];
+					frac += fracstep;
+				}
+			}
+
+			glTexImage2D(GL_TEXTURE_2D, 0, gl_tex_solid_format,
+					256, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE,
+					image32);
+		}
 	}
 	else
 	{
+		unsigned char image8[256 * 256];
 		unsigned char *dest;
 
 		for (i = 0; i < trows; i++)
@@ -375,29 +515,22 @@ Draw_StretchRaw(int x, int y, int w, int h, int cols, int rows, byte *data)
 			}
 		}
 
-		glTexImage2D(GL_TEXTURE_2D,
-				0,
-				GL_COLOR_INDEX8_EXT,
-				256, 256,
-				0,
-				GL_COLOR_INDEX,
-				GL_UNSIGNED_BYTE,
-				image8);
+		glTexImage2D(GL_TEXTURE_2D, 0, GL_COLOR_INDEX8_EXT, 256, 256,
+				0, GL_COLOR_INDEX, GL_UNSIGNED_BYTE, image8);
 	}
 
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 
-	glBegin(GL_QUADS);
-	glTexCoord2f(1.0 / 512.0, 1.0 / 512.0);
-	glVertex2f(x, y);
-	glTexCoord2f(511.0 / 512.0, 1.0 / 512.0);
-	glVertex2f(x + w, y);
-	glTexCoord2f(511.0 / 512.0, t);
-	glVertex2f(x + w, y + h);
-	glTexCoord2f(1.0 / 512.0, t);
-	glVertex2f(x, y + h);
-	glEnd();
+	glEnableClientState( GL_VERTEX_ARRAY );
+	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+
+	glVertexPointer( 2, GL_FLOAT, 0, vtx );
+	glTexCoordPointer( 2, GL_FLOAT, 0, tex );
+	glDrawArrays( GL_TRIANGLE_FAN, 0, 4 );
+
+	glDisableClientState( GL_VERTEX_ARRAY );
+	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
 }
 
 int
