@@ -43,11 +43,11 @@ qboolean R_Upload8(byte *data, int width, int height,
 		qboolean mipmap, qboolean is_sky);
 qboolean R_Upload32(unsigned *data, int width, int height, qboolean mipmap);
 
-int gl_solid_format = 3;
-int gl_alpha_format = 4;
+int gl_solid_format = GL_RGB;
+int gl_alpha_format = GL_RGBA;
 
-int gl_tex_solid_format = 3;
-int gl_tex_alpha_format = 4;
+int gl_tex_solid_format = GL_RGB;
+int gl_tex_alpha_format = GL_RGBA;
 
 int gl_filter_min = GL_LINEAR_MIPMAP_NEAREST;
 int gl_filter_max = GL_LINEAR;
@@ -78,7 +78,7 @@ typedef struct
 } gltmode_t;
 
 gltmode_t gl_alpha_modes[] = {
-	{"default", 4},
+	{"default", GL_RGBA},
 	{"GL_RGBA", GL_RGBA},
 	{"GL_RGBA8", GL_RGBA8},
 	{"GL_RGB5_A1", GL_RGB5_A1},
@@ -89,15 +89,12 @@ gltmode_t gl_alpha_modes[] = {
 #define NUM_GL_ALPHA_MODES (sizeof(gl_alpha_modes) / sizeof(gltmode_t))
 
 gltmode_t gl_solid_modes[] = {
-	{"default", 3},
+	{"default", GL_RGB},
 	{"GL_RGB", GL_RGB},
 	{"GL_RGB8", GL_RGB8},
 	{"GL_RGB5", GL_RGB5},
 	{"GL_RGB4", GL_RGB4},
 	{"GL_R3_G3_B2", GL_R3_G3_B2},
-#ifdef GL_RGB2_EXT
-	{"GL_RGB2", GL_RGB2_EXT},
-#endif
 };
 
 #define NUM_GL_SOLID_MODES (sizeof(gl_solid_modes) / sizeof(gltmode_t))
@@ -134,7 +131,7 @@ R_SetTexturePalette(unsigned palette[256])
 	int i;
 	unsigned char temptable[768];
 
-	if (qglColorTableEXT && gl_ext_palettedtexture->value)
+	if (gl_config.palettedtexture)
 	{
 		for (i = 0; i < 256; i++)
 		{
@@ -151,7 +148,7 @@ R_SetTexturePalette(unsigned palette[256])
 void
 R_EnableMultitexture(qboolean enable)
 {
-	if (!qglActiveTextureARB)
+	if (!gl_config.multitexture)
 	{
 		return;
 	}
@@ -178,7 +175,7 @@ R_SelectTexture(GLenum texture)
 {
 	int tmu;
 
-	if (!qglActiveTextureARB)
+	if (!gl_config.multitexture)
 	{
 		return;
 	}
@@ -198,12 +195,10 @@ R_SelectTexture(GLenum texture)
 	}
 
 	gl_state.currenttmu = tmu;
+	gl_state.currenttarget = texture;
 
-	if (qglActiveTextureARB)
-	{
-		qglActiveTextureARB(texture);
-		qglClientActiveTextureARB(texture);
-	}
+	qglActiveTextureARB(texture);
+	qglClientActiveTextureARB(texture);
 }
 
 void
@@ -240,7 +235,10 @@ R_Bind(int texnum)
 void
 R_MBind(GLenum target, int texnum)
 {
-	R_SelectTexture(target);
+	if (target != gl_state.currenttarget)
+	{
+		R_SelectTexture(target);
+	}
 
 	if (target == GL_TEXTURE0_ARB)
 	{
@@ -667,7 +665,7 @@ R_Upload32Native(unsigned *data, int width, int height, qboolean mipmap)
 
 
 qboolean
-R_Upload32Old(unsigned *data, int width, int height, qboolean mipmap)
+R_Upload32Soft(unsigned *data, int width, int height, qboolean mipmap)
 {
 	int samples;
 	unsigned scaled[256 * 256];
@@ -753,7 +751,7 @@ R_Upload32Old(unsigned *data, int width, int height, qboolean mipmap)
 	{
 		if (!mipmap)
 		{
-			if (qglColorTableEXT && gl_ext_palettedtexture->value &&
+			if (qglColorTableEXT && gl_palettedtexture->value &&
 				(samples == gl_solid_format))
 			{
 				uploaded_paletted = true;
@@ -783,7 +781,7 @@ R_Upload32Old(unsigned *data, int width, int height, qboolean mipmap)
 
 	R_LightScaleTexture(scaled, scaled_width, scaled_height, !mipmap);
 
-	if (qglColorTableEXT && gl_ext_palettedtexture->value &&
+	if (qglColorTableEXT && gl_palettedtexture->value &&
 		(samples == gl_solid_format))
 	{
 		uploaded_paletted = true;
@@ -824,7 +822,7 @@ R_Upload32Old(unsigned *data, int width, int height, qboolean mipmap)
 
 			miplevel++;
 
-			if (qglColorTableEXT && gl_ext_palettedtexture->value &&
+			if (qglColorTableEXT && gl_palettedtexture->value &&
 				(samples == gl_solid_format))
 			{
 				uploaded_paletted = true;
@@ -852,13 +850,13 @@ R_Upload32(unsigned *data, int width, int height, qboolean mipmap)
 {
 	qboolean res;
 
-	if (gl_config.tex_npot)
+	if (gl_config.npottextures)
 	{
 		res = R_Upload32Native(data, width, height, mipmap);
 	}
 	else
 	{
-		res = R_Upload32Old(data, width, height, mipmap);
+		res = R_Upload32Soft(data, width, height, mipmap);
 	}
 
 	if (mipmap)
@@ -898,7 +896,7 @@ R_Upload8(byte *data, int width, int height, qboolean mipmap, qboolean is_sky)
 		VID_Error(ERR_DROP, "R_Upload8: too large");
 	}
 
-	if (qglColorTableEXT && gl_ext_palettedtexture->value && is_sky)
+	if (gl_config.palettedtexture && is_sky)
 	{
 		glTexImage2D(GL_TEXTURE_2D, 0, GL_COLOR_INDEX8_EXT,
 				width, height, 0, GL_COLOR_INDEX, GL_UNSIGNED_BYTE,
@@ -961,6 +959,7 @@ R_LoadPic(char *name, byte *pic, int width, int realwidth,
 {
 	image_t *image;
 	int i;
+	qboolean nolerp = (strstr(Cvar_VariableString("gl_nolerp_list"), name) != NULL);
 
 	/* find a free image_t */
 	for (i = 0, image = gltextures; i < numgltextures; i++, image++)
@@ -1001,7 +1000,7 @@ R_LoadPic(char *name, byte *pic, int width, int realwidth,
 	}
 
 	/* load little pics into the scrap */
-	if ((image->type == it_pic) && (bits == 8) &&
+	if (!nolerp && (image->type == it_pic) && (bits == 8) &&
 		(image->width < 64) && (image->height < 64))
 	{
 		int x, y;
@@ -1078,6 +1077,12 @@ R_LoadPic(char *name, byte *pic, int width, int realwidth,
 		image->sh = 1;
 		image->tl = 0;
 		image->th = 1;
+
+		if (nolerp)
+		{
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+		}
 	}
 
 	return image;
@@ -1325,7 +1330,8 @@ void
 R_InitImages(void)
 {
 	int i, j;
-	float g = vid_gamma->value;
+	// use 1/gamma so higher value is brighter, to match HW gamma settings
+	float g = 1.0f/vid_gamma->value;
 
 	registration_sequence = 1;
 
@@ -1341,7 +1347,7 @@ R_InitImages(void)
 
 	Draw_GetPalette();
 
-	if (qglColorTableEXT)
+	if (gl_config.palettedtexture)
 	{
 		FS_LoadFile("pics/16to8.dat", (void **)&gl_state.d_16to8table);
 
