@@ -75,7 +75,6 @@ cvar_t *gl_drawworld;
 cvar_t *gl_speeds;
 cvar_t *gl_fullbright;
 cvar_t *gl_novis;
-cvar_t *gl_nocull;
 cvar_t *gl_lerpmodels;
 cvar_t *gl_lefthand;
 cvar_t *gl_farsee;
@@ -83,11 +82,6 @@ cvar_t *gl_farsee;
 cvar_t *gl_lightlevel;
 cvar_t *gl_overbrightbits;
 
-cvar_t *gl_nosubimage;
-cvar_t *gl_allow_software;
-
-cvar_t *gl_vertex_arrays;
-
 cvar_t *gl_particle_min_size;
 cvar_t *gl_particle_max_size;
 cvar_t *gl_particle_size;
@@ -95,14 +89,11 @@ cvar_t *gl_particle_att_a;
 cvar_t *gl_particle_att_b;
 cvar_t *gl_particle_att_c;
 
-cvar_t *gl_ext_swapinterval;
-cvar_t *gl_ext_palettedtexture;
-cvar_t *gl_ext_multitexture;
-cvar_t *gl_ext_pointparameters;
-cvar_t *gl_ext_compiled_vertex_array;
-cvar_t *gl_ext_mtexcombine;
+cvar_t *gl_palettedtexture;
+cvar_t *gl_multitexture;
+cvar_t *gl_pointparameters;
+cvar_t *gl_mtexcombine;
 
-cvar_t *gl_bitdepth;
 cvar_t *gl_drawbuffer;
 cvar_t *gl_lightmap;
 cvar_t *gl_shadows;
@@ -119,7 +110,6 @@ cvar_t *gl_modulate;
 cvar_t *gl_nobind;
 cvar_t *gl_round_down;
 cvar_t *gl_picmip;
-cvar_t *gl_skymip;
 cvar_t *gl_showtris;
 cvar_t *gl_ztrick;
 cvar_t *gl_zfix;
@@ -128,20 +118,23 @@ cvar_t *gl_clear;
 cvar_t *gl_cull;
 cvar_t *gl_polyblend;
 cvar_t *gl_flashblend;
-cvar_t *gl_playermip;
 cvar_t *gl_saturatelighting;
 cvar_t *gl_swapinterval;
 cvar_t *gl_texturemode;
 cvar_t *gl_texturealphamode;
 cvar_t *gl_texturesolidmode;
 cvar_t *gl_anisotropic;
-cvar_t *gl_anisotropic_avail;
 cvar_t *gl_lockpvs;
 cvar_t *gl_msaa_samples;
 
 cvar_t *vid_fullscreen;
 cvar_t *vid_gamma;
 
+cvar_t *gl_stereo;
+cvar_t *gl_stereo_separation;
+cvar_t *gl_stereo_anaglyph_colors;
+cvar_t *gl_stereo_convergence;
+
 /*
  * Returns true if the box is completely outside the frustom
  */
@@ -150,7 +143,7 @@ R_CullBox(vec3_t mins, vec3_t maxs)
 {
 	int i;
 
-	if (gl_nocull->value)
+	if (!gl_cull->value)
 	{
 		return false;
 	}
@@ -180,7 +173,7 @@ void
 R_DrawSpriteModel(entity_t *e)
 {
 	float alpha = 1.0F;
-	vec3_t point;
+    vec3_t point[4];
 	dsprframe_t *frame;
 	float *up, *right;
 	dsprite_t *psprite;
@@ -221,29 +214,34 @@ R_DrawSpriteModel(entity_t *e)
 		glDisable(GL_ALPHA_TEST);
 	}
 
-	glBegin(GL_QUADS);
+	GLfloat tex[] = {
+		0, 1,
+		0, 0,
+		1, 0,
+		1, 1
+	};
 
-	glTexCoord2f(0, 1);
-	VectorMA(e->origin, -frame->origin_y, up, point);
-	VectorMA(point, -frame->origin_x, right, point);
-	glVertex3fv(point);
+	VectorMA( e->origin, -frame->origin_y, up, point[0] );
+	VectorMA( point[0], -frame->origin_x, right, point[0] );
 
-	glTexCoord2f(0, 0);
-	VectorMA(e->origin, frame->height - frame->origin_y, up, point);
-	VectorMA(point, -frame->origin_x, right, point);
-	glVertex3fv(point);
+	VectorMA( e->origin, frame->height - frame->origin_y, up, point[1] );
+	VectorMA( point[1], -frame->origin_x, right, point[1] );
 
-	glTexCoord2f(1, 0);
-	VectorMA(e->origin, frame->height - frame->origin_y, up, point);
-	VectorMA(point, frame->width - frame->origin_x, right, point);
-	glVertex3fv(point);
+	VectorMA( e->origin, frame->height - frame->origin_y, up, point[2] );
+	VectorMA( point[2], frame->width - frame->origin_x, right, point[2] );
 
-	glTexCoord2f(1, 1);
-	VectorMA(e->origin, -frame->origin_y, up, point);
-	VectorMA(point, frame->width - frame->origin_x, right, point);
-	glVertex3fv(point);
+	VectorMA( e->origin, -frame->origin_y, up, point[3] );
+	VectorMA( point[3], frame->width - frame->origin_x, right, point[3] );
 
-	glEnd();
+	glEnableClientState( GL_VERTEX_ARRAY );
+	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+
+	glVertexPointer( 3, GL_FLOAT, 0, point );
+	glTexCoordPointer( 2, GL_FLOAT, 0, tex );
+	glDrawArrays( GL_TRIANGLE_FAN, 0, 4 );
+
+	glDisableClientState( GL_VERTEX_ARRAY );
+	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
 
 	glDisable(GL_ALPHA_TEST);
 	R_TexEnv(GL_REPLACE);
@@ -260,7 +258,6 @@ void
 R_DrawNullModel(void)
 {
 	vec3_t shadelight;
-	int i;
 
 	if (currententity->flags & RF_FULLBRIGHT)
 	{
@@ -275,29 +272,41 @@ R_DrawNullModel(void)
 	R_RotateForEntity(currententity);
 
 	glDisable(GL_TEXTURE_2D);
-	glColor3fv(shadelight);
+	glColor4f( shadelight[0], shadelight[1], shadelight[2], 1 );
 
-	glBegin(GL_TRIANGLE_FAN);
-	glVertex3f(0, 0, -16);
+    GLfloat vtxA[] = {
+        0, 0, -16,
+        16 * cos( 0 * M_PI / 2 ), 16 * sin( 0 * M_PI / 2 ), 0,
+        16 * cos( 1 * M_PI / 2 ), 16 * sin( 1 * M_PI / 2 ), 0,
+        16 * cos( 2 * M_PI / 2 ), 16 * sin( 2 * M_PI / 2 ), 0,
+        16 * cos( 3 * M_PI / 2 ), 16 * sin( 3 * M_PI / 2 ), 0,
+        16 * cos( 4 * M_PI / 2 ), 16 * sin( 4 * M_PI / 2 ), 0
+    };
 
-	for (i = 0; i <= 4; i++)
-	{
-		glVertex3f(16 * cos(i * M_PI / 2), 16 * sin(i * M_PI / 2), 0);
-	}
+    glEnableClientState( GL_VERTEX_ARRAY );
 
-	glEnd();
+    glVertexPointer( 3, GL_FLOAT, 0, vtxA );
+    glDrawArrays( GL_TRIANGLE_FAN, 0, 6 );
 
-	glBegin(GL_TRIANGLE_FAN);
-	glVertex3f(0, 0, 16);
+    glDisableClientState( GL_VERTEX_ARRAY );
 
-	for (i = 4; i >= 0; i--)
-	{
-		glVertex3f(16 * cos(i * M_PI / 2), 16 * sin(i * M_PI / 2), 0);
-	}
+	GLfloat vtxB[] = {
+		0, 0, 16,
+		16 * cos( 4 * M_PI / 2 ), 16 * sin( 4 * M_PI / 2 ), 0,
+		16 * cos( 3 * M_PI / 2 ), 16 * sin( 3 * M_PI / 2 ), 0,
+		16 * cos( 2 * M_PI / 2 ), 16 * sin( 2 * M_PI / 2 ), 0,
+		16 * cos( 1 * M_PI / 2 ), 16 * sin( 1 * M_PI / 2 ), 0,
+		16 * cos( 0 * M_PI / 2 ), 16 * sin( 0 * M_PI / 2 ), 0
+	};
 
-	glEnd();
+	glEnableClientState( GL_VERTEX_ARRAY );
 
-	glColor3f(1, 1, 1);
+	glVertexPointer( 3, GL_FLOAT, 0, vtxB );
+	glDrawArrays( GL_TRIANGLE_FAN, 0, 6 );
+
+	glDisableClientState( GL_VERTEX_ARRAY );
+
+	glColor4f(1, 1, 1, 1);
 	glPopMatrix();
 	glEnable(GL_TEXTURE_2D);
 }
@@ -412,24 +421,31 @@ R_DrawParticles2(int num_particles, const particle_t particles[],
 	vec3_t up, right;
 	float scale;
 	byte color[4];
-
+ 
+	GLfloat vtx[3*num_particles*3];
+	GLfloat tex[2*num_particles*3];
+	GLfloat clr[4*num_particles*3];
+	unsigned int index_vtx = 0;
+	unsigned int index_tex = 0;
+	unsigned int index_clr = 0;
+	unsigned int j;
+ 
 	R_Bind(r_particletexture->texnum);
 	glDepthMask(GL_FALSE); /* no z buffering */
 	glEnable(GL_BLEND);
 	R_TexEnv(GL_MODULATE);
-	glBegin(GL_TRIANGLES);
 
-	VectorScale(vup, 1.5, up);
-	VectorScale(vright, 1.5, right);
+	VectorScale( vup, 1.5, up );
+	VectorScale( vright, 1.5, right );
 
-	for (p = particles, i = 0; i < num_particles; i++, p++)
+	for ( p = particles, i = 0; i < num_particles; i++, p++ )
 	{
 		/* hack a scale up to keep particles from disapearing */
-		scale = (p->origin[0] - r_origin[0]) * vpn[0] +
-				(p->origin[1] - r_origin[1]) * vpn[1] +
-				(p->origin[2] - r_origin[2]) * vpn[2];
+		scale = ( p->origin [ 0 ] - r_origin [ 0 ] ) * vpn [ 0 ] +
+			( p->origin [ 1 ] - r_origin [ 1 ] ) * vpn [ 1 ] +
+			( p->origin [ 2 ] - r_origin [ 2 ] ) * vpn [ 2 ];
 
-		if (scale < 20)
+		if ( scale < 20 )
 		{
 			scale = 1;
 		}
@@ -438,26 +454,54 @@ R_DrawParticles2(int num_particles, const particle_t particles[],
 			scale = 1 + scale * 0.004;
 		}
 
-		*(int *)color = colortable[p->color];
-		color[3] = p->alpha * 255;
+		*(int *) color = colortable [ p->color ];
 
-		glColor4ubv(color);
+		for (j=0; j<3; j++) // Copy the color for each point
+		{
+			clr[index_clr++] = color[0]/255.0f;
+			clr[index_clr++] = color[1]/255.0f;
+			clr[index_clr++] = color[2]/255.0f;
+			clr[index_clr++] = p->alpha;
+		}
 
-		glTexCoord2f(0.0625, 0.0625);
-		glVertex3fv(p->origin);
+		// point 0
+		tex[index_tex++] = 0.0625f;
+		tex[index_tex++] = 0.0625f;
 
-		glTexCoord2f(1.0625, 0.0625);
-		glVertex3f(p->origin[0] + up[0] * scale,
-				p->origin[1] + up[1] * scale,
-				p->origin[2] + up[2] * scale);
+		vtx[index_vtx++] = p->origin[0];
+		vtx[index_vtx++] = p->origin[1];
+		vtx[index_vtx++] = p->origin[2];
 
-		glTexCoord2f(0.0625, 1.0625);
-		glVertex3f(p->origin[0] + right[0] * scale,
-				p->origin[1] + right[1] * scale,
-				p->origin[2] + right[2] * scale);
+		// point 1
+		tex[index_tex++] = 1.0625f;
+		tex[index_tex++] = 0.0625f;
+
+		vtx[index_vtx++] = p->origin [ 0 ] + up [ 0 ] * scale;
+		vtx[index_vtx++] = p->origin [ 1 ] + up [ 1 ] * scale;
+		vtx[index_vtx++] = p->origin [ 2 ] + up [ 2 ] * scale;
+
+		// point 2
+		tex[index_tex++] = 0.0625f;
+		tex[index_tex++] = 1.0625f;
+
+		vtx[index_vtx++] = p->origin [ 0 ] + right [ 0 ] * scale;
+		vtx[index_vtx++] = p->origin [ 1 ] + right [ 1 ] * scale;
+		vtx[index_vtx++] = p->origin [ 2 ] + right [ 2 ] * scale;
 	}
 
-	glEnd();
+	glEnableClientState( GL_VERTEX_ARRAY );
+	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+	glEnableClientState( GL_COLOR_ARRAY );
+
+	glVertexPointer( 3, GL_FLOAT, 0, vtx );
+	glTexCoordPointer( 2, GL_FLOAT, 0, tex );
+	glColorPointer( 4, GL_FLOAT, 0, clr );
+	glDrawArrays( GL_TRIANGLES, 0, num_particles*3 );
+
+	glDisableClientState( GL_VERTEX_ARRAY );
+	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
+	glDisableClientState( GL_COLOR_ARRAY );
+ 
 	glDisable(GL_BLEND);
 	glColor4f(1, 1, 1, 1);
 	glDepthMask(1); /* back to normal Z buffering */
@@ -467,34 +511,51 @@ R_DrawParticles2(int num_particles, const particle_t particles[],
 void
 R_DrawParticles(void)
 {
-	if (gl_ext_pointparameters->value && qglPointParameterfEXT)
+	qboolean stereo_split_tb = ((gl_state.stereo_mode == STEREO_SPLIT_VERTICAL) && gl_state.camera_separation);
+	qboolean stereo_split_lr = ((gl_state.stereo_mode == STEREO_SPLIT_HORIZONTAL) && gl_state.camera_separation);
+
+	if (gl_config.pointparameters && !(stereo_split_tb || stereo_split_lr))
 	{
 		int i;
 		unsigned char color[4];
 		const particle_t *p;
-
+ 
+		GLfloat vtx[3*r_newrefdef.num_particles];
+		GLfloat clr[4*r_newrefdef.num_particles];
+		unsigned int index_vtx = 0;
+		unsigned int index_clr = 0;
+  
 		glDepthMask(GL_FALSE);
 		glEnable(GL_BLEND);
 		glDisable(GL_TEXTURE_2D);
 
 		glPointSize(LittleFloat(gl_particle_size->value));
 
-		glBegin(GL_POINTS);
-
-		for (i = 0, p = r_newrefdef.particles;
-			 i < r_newrefdef.num_particles;
-			 i++, p++)
+		for ( i = 0, p = r_newrefdef.particles; i < r_newrefdef.num_particles; i++, p++ )
 		{
-			*(int *)color = d_8to24table[p->color & 0xFF];
-			color[3] = p->alpha * 255;
-			glColor4ubv(color);
-			glVertex3fv(p->origin);
+			*(int *) color = d_8to24table [ p->color & 0xFF ];
+			clr[index_clr++] = color[0]/255.0f;
+			clr[index_clr++] = color[1]/255.0f;
+			clr[index_clr++] = color[2]/255.0f;
+			clr[index_clr++] = p->alpha;
+
+			vtx[index_vtx++] = p->origin[0];
+			vtx[index_vtx++] = p->origin[1];
+			vtx[index_vtx++] = p->origin[2];
 		}
 
-		glEnd();
+		glEnableClientState( GL_VERTEX_ARRAY );
+		glEnableClientState( GL_COLOR_ARRAY );
+
+		glVertexPointer( 3, GL_FLOAT, 0, vtx );
+		glColorPointer( 4, GL_FLOAT, 0, clr );
+		glDrawArrays( GL_POINTS, 0, r_newrefdef.num_particles );
+
+		glDisableClientState( GL_VERTEX_ARRAY );
+		glDisableClientState( GL_COLOR_ARRAY );
 
 		glDisable(GL_BLEND);
-		glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
+		glColor4f( 1, 1, 1, 1 );
 		glDepthMask(GL_TRUE);
 		glEnable(GL_TEXTURE_2D);
 	}
@@ -528,15 +589,21 @@ R_PolyBlend(void)
 	glRotatef(-90, 1, 0, 0); /* put Z going up */
 	glRotatef(90, 0, 0, 1); /* put Z going up */
 
-	glColor4fv(v_blend);
+	glColor4f( v_blend[0], v_blend[1], v_blend[2], v_blend[3] );
 
-	glBegin(GL_QUADS);
+	GLfloat vtx[] = {
+		10, 100, 100,
+		10, -100, 100,
+		10, -100, -100,
+		10, 100, -100
+	};
 
-	glVertex3f(10, 100, 100);
-	glVertex3f(10, -100, 100);
-	glVertex3f(10, -100, -100);
-	glVertex3f(10, 100, -100);
-	glEnd();
+	glEnableClientState( GL_VERTEX_ARRAY );
+
+	glVertexPointer( 3, GL_FLOAT, 0, vtx );
+	glDrawArrays( GL_TRIANGLE_FAN, 0, 4 );
+
+	glDisableClientState( GL_VERTEX_ARRAY );
 
 	glDisable(GL_BLEND);
 	glEnable(GL_TEXTURE_2D);
@@ -678,8 +745,8 @@ R_MYgluPerspective(GLdouble fovy, GLdouble aspect,
 	xmin = ymin * aspect;
 	xmax = ymax * aspect;
 
-	xmin += -(2 * gl_state.camera_separation) / zNear;
-	xmax += -(2 * gl_state.camera_separation) / zNear;
+	xmin += - gl_stereo_convergence->value * (2 * gl_state.camera_separation) / zNear;
+	xmax += - gl_stereo_convergence->value * (2 * gl_state.camera_separation) / zNear;
 
 	glFrustum(xmin, xmax, ymin, ymax, zNear, zFar);
 }
@@ -700,6 +767,20 @@ R_SetupGL(void)
 	w = x2 - x;
 	h = y - y2;
 
+	qboolean drawing_left_eye = gl_state.camera_separation < 0;
+	qboolean stereo_split_tb = ((gl_state.stereo_mode == STEREO_SPLIT_VERTICAL) && gl_state.camera_separation);
+	qboolean stereo_split_lr = ((gl_state.stereo_mode == STEREO_SPLIT_HORIZONTAL) && gl_state.camera_separation);
+
+	if(stereo_split_lr) {
+		w = w / 2;
+		x = drawing_left_eye ? (x / 2) : (x + vid.width) / 2;
+	}
+
+	if(stereo_split_tb) {
+		h = h / 2;
+		y2 = drawing_left_eye ? (y2 + vid.height) / 2 : (y2 / 2);
+	}
+
 	glViewport(x, y2, w, h);
 
 	/* set up projection matrix */
@@ -749,13 +830,20 @@ R_SetupGL(void)
 void
 R_Clear(void)
 {
+	// Check whether the stencil buffer needs clearing, and do so if need be.
+	GLbitfield stencilFlags = 0;
+	if (gl_state.stereo_mode >= STEREO_MODE_ROW_INTERLEAVED && gl_state.stereo_mode <= STEREO_MODE_PIXEL_INTERLEAVED) {
+		glClearStencil(0);
+		stencilFlags |= GL_STENCIL_BUFFER_BIT;
+	}
+
 	if (gl_ztrick->value)
 	{
 		static int trickframe;
 
 		if (gl_clear->value)
 		{
-			glClear(GL_COLOR_BUFFER_BIT);
+			glClear(GL_COLOR_BUFFER_BIT | stencilFlags);
 		}
 
 		trickframe++;
@@ -777,11 +865,11 @@ R_Clear(void)
 	{
 		if (gl_clear->value)
 		{
-			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+			glClear(GL_COLOR_BUFFER_BIT | stencilFlags | GL_DEPTH_BUFFER_BIT);
 		}
 		else
 		{
-			glClear(GL_DEPTH_BUFFER_BIT);
+			glClear(GL_DEPTH_BUFFER_BIT | stencilFlags);
 		}
 
 		gldepthmin = 0;
@@ -817,12 +905,158 @@ R_Flash(void)
 	R_PolyBlend();
 }
 
+void
+R_SetGL2D(void)
+{
+	int x, w, y, h;
+	/* set 2D virtual screen size */
+	qboolean drawing_left_eye = gl_state.camera_separation < 0;
+	qboolean stereo_split_tb = ((gl_state.stereo_mode == STEREO_SPLIT_VERTICAL) && gl_state.camera_separation);
+	qboolean stereo_split_lr = ((gl_state.stereo_mode == STEREO_SPLIT_HORIZONTAL) && gl_state.camera_separation);
+
+	x = 0;
+	w = vid.width;
+	y = 0;
+	h = vid.height;
+
+	if(stereo_split_lr) {
+		w =  w / 2;
+		x = drawing_left_eye ? 0 : w;
+	}
+
+	if(stereo_split_tb) {
+		h =  h / 2;
+		y = drawing_left_eye ? h : 0;
+	}
+
+	glViewport(x, y, w, h);
+	glMatrixMode(GL_PROJECTION);
+	glLoadIdentity();
+	glOrtho(0, vid.width, vid.height, 0, -99999, 99999);
+	glMatrixMode(GL_MODELVIEW);
+	glLoadIdentity();
+	glDisable(GL_DEPTH_TEST);
+	glDisable(GL_CULL_FACE);
+	glDisable(GL_BLEND);
+	glEnable(GL_ALPHA_TEST);
+	glColor4f(1, 1, 1, 1);
+}
+
 /*
  * r_newrefdef must be set before the first call
  */
 void
 R_RenderView(refdef_t *fd)
 {
+	if ((gl_state.stereo_mode != STEREO_MODE_NONE) && gl_state.camera_separation) {
+
+		qboolean drawing_left_eye = gl_state.camera_separation < 0;
+		switch (gl_state.stereo_mode) {
+			case STEREO_MODE_ANAGLYPH:
+				{
+
+					// Work out the colour for each eye.
+					int anaglyph_colours[] = { 0x4, 0x3 }; // Left = red, right = cyan.
+
+					if (strlen(gl_stereo_anaglyph_colors->string) == 2) {
+						int eye, colour, missing_bits;
+						// Decode the colour name from its character.
+						for (eye = 0; eye < 2; ++eye) {
+							colour = 0;
+							switch (toupper(gl_stereo_anaglyph_colors->string[eye])) {
+								case 'B': ++colour; // 001 Blue
+								case 'G': ++colour; // 010 Green
+								case 'C': ++colour; // 011 Cyan
+								case 'R': ++colour; // 100 Red
+								case 'M': ++colour; // 101 Magenta
+								case 'Y': ++colour; // 110 Yellow
+									anaglyph_colours[eye] = colour;
+									break;
+							}
+						}
+						// Fill in any missing bits.
+						missing_bits = ~(anaglyph_colours[0] | anaglyph_colours[1]) & 0x3;
+						for (eye = 0; eye < 2; ++eye) {
+							anaglyph_colours[eye] |= missing_bits;
+						}
+					}
+
+					// Set the current colour.
+					glColorMask(
+						!!(anaglyph_colours[drawing_left_eye] & 0x4),
+						!!(anaglyph_colours[drawing_left_eye] & 0x2),
+						!!(anaglyph_colours[drawing_left_eye] & 0x1),
+						GL_TRUE
+					);
+				}
+				break;
+			case STEREO_MODE_ROW_INTERLEAVED:
+			case STEREO_MODE_COLUMN_INTERLEAVED:
+			case STEREO_MODE_PIXEL_INTERLEAVED:
+				{
+					qboolean flip_eyes = true;
+					int client_x, client_y;
+
+					//GLimp_GetClientAreaOffset(&client_x, &client_y);
+					client_x = 0;
+					client_y = 0;
+
+					R_SetGL2D();
+
+					glEnable(GL_STENCIL_TEST);
+					glStencilMask(GL_TRUE);
+					glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
+
+					glStencilOp(GL_REPLACE, GL_KEEP, GL_KEEP);
+					glStencilFunc(GL_NEVER, 0, 1);
+
+					glBegin(GL_QUADS);
+					{
+						glVertex2i(0, 0);
+						glVertex2i(vid.width, 0);
+						glVertex2i(vid.width, vid.height);
+						glVertex2i(0, vid.height);
+					}
+					glEnd();
+
+					glStencilOp(GL_INVERT, GL_KEEP, GL_KEEP);
+					glStencilFunc(GL_NEVER, 1, 1);
+
+					glBegin(GL_LINES);
+					{
+						if (gl_state.stereo_mode == STEREO_MODE_ROW_INTERLEAVED || gl_state.stereo_mode == STEREO_MODE_PIXEL_INTERLEAVED) {
+							int y;
+							for (y = 0; y <= vid.height; y += 2) {
+								glVertex2f(0, y - 0.5f);
+								glVertex2f(vid.width, y - 0.5f);
+							}
+							flip_eyes ^= (client_y & 1);
+						}
+
+						if (gl_state.stereo_mode == STEREO_MODE_COLUMN_INTERLEAVED || gl_state.stereo_mode == STEREO_MODE_PIXEL_INTERLEAVED) {
+							int x;
+							for (x = 0; x <= vid.width; x += 2) {
+								glVertex2f(x - 0.5f, 0);
+								glVertex2f(x - 0.5f, vid.height);
+							}
+							flip_eyes ^= (client_x & 1);
+						}
+					}
+					glEnd();
+
+					glStencilMask(GL_FALSE);
+					glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
+
+					glStencilFunc(GL_EQUAL, drawing_left_eye ^ flip_eyes, 1);
+					glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
+				}
+				break;
+			default:
+				break;
+		}
+	}
+
+
 	if (gl_norefresh->value)
 	{
 		return;
@@ -874,23 +1108,36 @@ R_RenderView(refdef_t *fd)
 				c_brush_polys, c_alias_polys, c_visible_textures,
 				c_visible_lightmaps);
 	}
+
+	switch (gl_state.stereo_mode) {
+		case STEREO_MODE_ANAGLYPH:
+			glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
+			break;
+		case STEREO_MODE_ROW_INTERLEAVED:
+		case STEREO_MODE_COLUMN_INTERLEAVED:
+		case STEREO_MODE_PIXEL_INTERLEAVED:
+			glDisable(GL_STENCIL_TEST);
+			break;
+		default:
+			break;
+	}
 }
 
-void
-R_SetGL2D(void)
-{
-	/* set 2D virtual screen size */
-	glViewport(0, 0, vid.width, vid.height);
-	glMatrixMode(GL_PROJECTION);
-	glLoadIdentity();
-	glOrtho(0, vid.width, vid.height, 0, -99999, 99999);
-	glMatrixMode(GL_MODELVIEW);
-	glLoadIdentity();
-	glDisable(GL_DEPTH_TEST);
-	glDisable(GL_CULL_FACE);
-	glDisable(GL_BLEND);
-	glEnable(GL_ALPHA_TEST);
-	glColor4f(1, 1, 1, 1);
+enum opengl_special_buffer_modes GL_GetSpecialBufferModeForStereoMode(enum stereo_modes stereo_mode) {
+	switch (stereo_mode) {
+		case STEREO_MODE_NONE:
+		case STEREO_SPLIT_HORIZONTAL:
+		case STEREO_SPLIT_VERTICAL:
+		case STEREO_MODE_ANAGLYPH:
+			return OPENGL_SPECIAL_BUFFER_MODE_NONE;
+		case STEREO_MODE_OPENGL:
+			return OPENGL_SPECIAL_BUFFER_MODE_STEREO;
+		case STEREO_MODE_ROW_INTERLEAVED:
+		case STEREO_MODE_COLUMN_INTERLEAVED:
+		case STEREO_MODE_PIXEL_INTERLEAVED:
+			return OPENGL_SPECIAL_BUFFER_MODE_STENCIL;
+	}
+	return OPENGL_SPECIAL_BUFFER_MODE_NONE;
 }
 
 void
@@ -950,15 +1197,11 @@ R_Register(void)
 	gl_drawentities = Cvar_Get("gl_drawentities", "1", 0);
 	gl_drawworld = Cvar_Get("gl_drawworld", "1", 0);
 	gl_novis = Cvar_Get("gl_novis", "0", 0);
-	gl_nocull = Cvar_Get("gl_nocull", "0", 0);
 	gl_lerpmodels = Cvar_Get("gl_lerpmodels", "1", 0);
 	gl_speeds = Cvar_Get("gl_speeds", "0", 0);
 
 	gl_lightlevel = Cvar_Get("gl_lightlevel", "0", 0);
-	gl_overbrightbits = Cvar_Get("gl_overbrightbits", "2", CVAR_ARCHIVE);
-
-	gl_nosubimage = Cvar_Get("gl_nosubimage", "0", 0);
-	gl_allow_software = Cvar_Get("gl_allow_software", "0", 0);
+	gl_overbrightbits = Cvar_Get("gl_overbrightbits", "0", CVAR_ARCHIVE);
 
 	gl_particle_min_size = Cvar_Get("gl_particle_min_size", "2", CVAR_ARCHIVE);
 	gl_particle_max_size = Cvar_Get("gl_particle_max_size", "40", CVAR_ARCHIVE);
@@ -968,7 +1211,6 @@ R_Register(void)
 	gl_particle_att_c = Cvar_Get("gl_particle_att_c", "0.01", CVAR_ARCHIVE);
 
 	gl_modulate = Cvar_Get("gl_modulate", "1", CVAR_ARCHIVE);
-	gl_bitdepth = Cvar_Get("gl_bitdepth", "0", 0);
 	gl_mode = Cvar_Get("gl_mode", "4", CVAR_ARCHIVE);
 	gl_lightmap = Cvar_Get("gl_lightmap", "0", 0);
 	gl_shadows = Cvar_Get("gl_shadows", "0", CVAR_ARCHIVE);
@@ -977,7 +1219,6 @@ R_Register(void)
 	gl_nobind = Cvar_Get("gl_nobind", "0", 0);
 	gl_round_down = Cvar_Get("gl_round_down", "1", 0);
 	gl_picmip = Cvar_Get("gl_picmip", "0", 0);
-	gl_skymip = Cvar_Get("gl_skymip", "0", 0);
 	gl_showtris = Cvar_Get("gl_showtris", "0", 0);
 	gl_ztrick = Cvar_Get("gl_ztrick", "0", 0);
 	gl_zfix = Cvar_Get("gl_zfix", "0", 0);
@@ -986,23 +1227,17 @@ R_Register(void)
 	gl_cull = Cvar_Get("gl_cull", "1", 0);
 	gl_polyblend = Cvar_Get("gl_polyblend", "1", 0);
 	gl_flashblend = Cvar_Get("gl_flashblend", "0", 0);
-	gl_playermip = Cvar_Get("gl_playermip", "0", 0);
 
 	gl_texturemode = Cvar_Get("gl_texturemode", "GL_LINEAR_MIPMAP_NEAREST", CVAR_ARCHIVE);
 	gl_texturealphamode = Cvar_Get("gl_texturealphamode", "default", CVAR_ARCHIVE);
 	gl_texturesolidmode = Cvar_Get("gl_texturesolidmode", "default", CVAR_ARCHIVE);
 	gl_anisotropic = Cvar_Get("gl_anisotropic", "0", CVAR_ARCHIVE);
-	gl_anisotropic_avail = Cvar_Get("gl_anisotropic_avail", "0", 0);
 	gl_lockpvs = Cvar_Get("gl_lockpvs", "0", 0);
 
-	gl_vertex_arrays = Cvar_Get("gl_vertex_arrays", "0", CVAR_ARCHIVE);
-
-	gl_ext_swapinterval = Cvar_Get("gl_ext_swapinterval", "1", CVAR_ARCHIVE);
-	gl_ext_palettedtexture = Cvar_Get("gl_ext_palettedtexture", "0", CVAR_ARCHIVE);
-	gl_ext_multitexture = Cvar_Get("gl_ext_multitexture", "1", CVAR_ARCHIVE);
-	gl_ext_pointparameters = Cvar_Get("gl_ext_pointparameters", "1", CVAR_ARCHIVE);
-	gl_ext_compiled_vertex_array = Cvar_Get("gl_ext_compiled_vertex_array", "1", CVAR_ARCHIVE);
-	gl_ext_mtexcombine = Cvar_Get("gl_ext_mtexcombine", "1", CVAR_ARCHIVE);
+	gl_palettedtexture = Cvar_Get("gl_palettedtexture", "0", CVAR_ARCHIVE);
+	gl_multitexture = Cvar_Get("gl_multitexture", "0", CVAR_ARCHIVE);
+	gl_pointparameters = Cvar_Get("gl_pointparameters", "1", CVAR_ARCHIVE);
+	gl_mtexcombine = Cvar_Get("gl_mtexcombine", "1", CVAR_ARCHIVE);
 
 	gl_drawbuffer = Cvar_Get("gl_drawbuffer", "GL_BACK", 0);
 	gl_swapinterval = Cvar_Get("gl_swapinterval", "1", CVAR_ARCHIVE);
@@ -1018,6 +1253,12 @@ R_Register(void)
 
 	gl_retexturing = Cvar_Get("gl_retexturing", "1", CVAR_ARCHIVE);
 
+
+	gl_stereo = Cvar_Get( "gl_stereo", "0", CVAR_ARCHIVE );
+	gl_stereo_separation = Cvar_Get( "gl_stereo_separation", "-0.4", CVAR_ARCHIVE );
+	gl_stereo_anaglyph_colors = Cvar_Get( "gl_stereo_anaglyph_colors", "rc", CVAR_ARCHIVE );
+	gl_stereo_convergence = Cvar_Get( "gl_stereo_convergence", "1", CVAR_ARCHIVE );
+
 	Cmd_AddCommand("imagelist", R_ImageList_f);
 	Cmd_AddCommand("screenshot", R_ScreenShot);
 	Cmd_AddCommand("modellist", Mod_Modellist_f);
@@ -1086,9 +1327,6 @@ R_SetMode(void)
 int
 R_Init(void *hinstance, void *hWnd)
 {
-	char renderer_buffer[1000];
-	char vendor_buffer[1000];
-	int err;
 	int j;
 	extern float r_turbsin[256];
 
@@ -1128,6 +1366,7 @@ R_Init(void *hinstance, void *hWnd)
 
 	/* set our "safe" mode */
 	gl_state.prev_mode = 4;
+	gl_state.stereo_mode = gl_stereo->value;
 
 	/* create the window and set up the context */
 	if (!R_SetMode())
@@ -1139,161 +1378,190 @@ R_Init(void *hinstance, void *hWnd)
 
 	VID_MenuInit();
 
+	// --------
+
 	/* get our various GL strings */
 	VID_Printf(PRINT_ALL, "\nOpenGL setting:\n", gl_config.vendor_string);
+
 	gl_config.vendor_string = (char *)glGetString(GL_VENDOR);
 	VID_Printf(PRINT_ALL, "GL_VENDOR: %s\n", gl_config.vendor_string);
+
 	gl_config.renderer_string = (char *)glGetString(GL_RENDERER);
 	VID_Printf(PRINT_ALL, "GL_RENDERER: %s\n", gl_config.renderer_string);
+
 	gl_config.version_string = (char *)glGetString(GL_VERSION);
 	VID_Printf(PRINT_ALL, "GL_VERSION: %s\n", gl_config.version_string);
+
 	gl_config.extensions_string = (char *)glGetString(GL_EXTENSIONS);
 	VID_Printf(PRINT_ALL, "GL_EXTENSIONS: %s\n", gl_config.extensions_string);
 
-	Q_strlcpy(renderer_buffer, gl_config.renderer_string, sizeof(renderer_buffer));
-	Q_strlwr(renderer_buffer);
+	sscanf(gl_config.version_string, "%d.%d", &gl_config.major_version, &gl_config.minor_version);
 
-	Q_strlcpy(vendor_buffer, gl_config.vendor_string, sizeof(vendor_buffer));
-	Q_strlwr(vendor_buffer);
+	if (gl_config.major_version == 1)
+	{
+		if (gl_config.minor_version < 4)
+		{
+			QGL_Shutdown();
+			VID_Printf(PRINT_ALL, "Support for OpenGL 1.4 is not available\n");
 
-	Cvar_Set("scr_drawall", "0");
-	gl_config.allow_cds = true;
+			return -1;
+		}
+	}
 
 	VID_Printf(PRINT_ALL, "\n\nProbing for OpenGL extensions:\n");
 
-	/* grab extensions */
-	if (strstr(gl_config.extensions_string, "GL_EXT_compiled_vertex_array"))
+	// ----
+
+	/* Point parameters */
+	VID_Printf(PRINT_ALL, " - Point parameters: ");
+
+	if (strstr(gl_config.extensions_string, "GL_ARB_point_parameters"))
 	{
-		VID_Printf(PRINT_ALL, "...using GL_EXT_compiled_vertex_array\n");
-		qglLockArraysEXT = ( void * ) GLimp_GetProcAddress ( "glLockArraysEXT" );
-		qglUnlockArraysEXT = ( void * ) GLimp_GetProcAddress ( "glUnlockArraysEXT" );
-	}
-	else
-	{
-		VID_Printf(PRINT_ALL, "...GL_EXT_compiled_vertex_array not found\n");
+			qglPointParameterfARB = (void (APIENTRY *)(GLenum, GLfloat))GLimp_GetProcAddress ( "glPointParameterfARB" );
+			qglPointParameterfvARB = (void (APIENTRY *)(GLenum, const GLfloat *))GLimp_GetProcAddress ( "glPointParameterfvARB" );
 	}
 
-	if (strstr(gl_config.extensions_string, "GL_EXT_point_parameters"))
+	gl_config.pointparameters = false;
+
+	if (gl_pointparameters->value)
 	{
-		if (gl_ext_pointparameters->value)
+		if (qglPointParameterfARB && qglPointParameterfvARB)
 		{
-			VID_Printf(PRINT_ALL, "...using GL_EXT_point_parameters\n");
-			qglPointParameterfEXT = (void (APIENTRY *)(GLenum, GLfloat))
-				GLimp_GetProcAddress ( "glPointParameterfEXT" );
-			qglPointParameterfvEXT = (void (APIENTRY *)(GLenum, const GLfloat *))
-				GLimp_GetProcAddress ( "glPointParameterfvEXT" );
+			gl_config.pointparameters = true;
+			VID_Printf(PRINT_ALL, "Okay\n");
 		}
 		else
 		{
-			VID_Printf(PRINT_ALL, "...ignoring GL_EXT_point_parameters\n");
+			VID_Printf(PRINT_ALL, "Failed\n");
 		}
 	}
 	else
 	{
-		VID_Printf(PRINT_ALL, "...GL_EXT_point_parameters not found\n");
+		VID_Printf(PRINT_ALL, "Disabled\n");
 	}
 
-	if (!qglColorTableEXT &&
-		strstr(gl_config.extensions_string, "GL_EXT_paletted_texture") &&
+	// ----
+
+	/* Paletted texture */
+	VID_Printf(PRINT_ALL, " - Paletted texture: ");
+
+	if (strstr(gl_config.extensions_string, "GL_EXT_paletted_texture") &&
 		strstr(gl_config.extensions_string, "GL_EXT_shared_texture_palette"))
 	{
-		if (gl_ext_palettedtexture->value)
+			qglColorTableEXT = (void (APIENTRY *)(GLenum, GLenum, GLsizei, GLenum, GLenum, const GLvoid * ))
+					GLimp_GetProcAddress ("glColorTableEXT");
+	}
+
+	gl_config.palettedtexture = false;
+
+	if (gl_palettedtexture->value)
+	{
+		if (qglColorTableEXT)
 		{
-			VID_Printf(PRINT_ALL, "...using GL_EXT_shared_texture_palette\n");
-			qglColorTableEXT =
-				(void (APIENTRY *)(GLenum, GLenum, GLsizei, GLenum, GLenum,
-						const GLvoid * ) ) GLimp_GetProcAddress ("glColorTableEXT");
+			gl_config.palettedtexture = true;
+			VID_Printf(PRINT_ALL, "Okay\n");
 		}
 		else
 		{
-			VID_Printf(PRINT_ALL, "...ignoring GL_EXT_shared_texture_palette\n");
+			VID_Printf(PRINT_ALL, "Failed\n");
 		}
 	}
 	else
 	{
-		VID_Printf(PRINT_ALL, "...GL_EXT_shared_texture_palette not found\n");
+		VID_Printf(PRINT_ALL, "Disabled\n");
 	}
 
+	// ----
+
+	/* Multitexturing */
+	VID_Printf(PRINT_ALL, " - Multitexturing: ");
+
 	if (strstr(gl_config.extensions_string, "GL_ARB_multitexture"))
 	{
-		if (gl_ext_multitexture->value)
+		qglMultiTexCoord2fARB = (void *)GLimp_GetProcAddress("glMultiTexCoord2fARB");
+		qglMultiTexCoord2fvARB = (void *)GLimp_GetProcAddress("glMultiTexCoord2fvARB");
+		qglActiveTextureARB = (void *)GLimp_GetProcAddress("glActiveTextureARB");
+		qglClientActiveTextureARB = (void *)GLimp_GetProcAddress("glClientActiveTextureARB");
+	}
+
+	gl_config.multitexture = false;
+
+	if (gl_multitexture->value)
+	{
+		if (qglMultiTexCoord2fARB && qglMultiTexCoord2fvARB && qglActiveTextureARB && qglClientActiveTextureARB)
 		{
-			VID_Printf(PRINT_ALL, "...using GL_ARB_multitexture\n");
-			qglMultiTexCoord2fARB = ( void * ) GLimp_GetProcAddress ( "glMultiTexCoord2fARB" );
-			qglActiveTextureARB = ( void * ) GLimp_GetProcAddress ( "glActiveTextureARB" );
-			qglClientActiveTextureARB = ( void * ) GLimp_GetProcAddress ( "glClientActiveTextureARB" );
+			gl_config.multitexture = true;
+			VID_Printf(PRINT_ALL, "Okay\n");
 		}
 		else
 		{
-			VID_Printf(PRINT_ALL, "...ignoring GL_ARB_multitexture\n");
+			VID_Printf(PRINT_ALL, "Failed\n");
 		}
 	}
 	else
 	{
-		VID_Printf(PRINT_ALL, "...GL_ARB_multitexture not found\n");
+		VID_Printf(PRINT_ALL, "Disabled\n");
 	}
 
-	gl_config.anisotropic = false;
+	// ----
+
+	/* Multi texturing combine */
+	VID_Printf(PRINT_ALL, " - Multi texturing combine: ");
+
+	if (strstr(gl_config.extensions_string, "GL_ARB_texture_env_combine") && gl_config.multitexture)
+	{
+		if (gl_mtexcombine->value)
+		{
+			gl_config.mtexcombine = true;
+			VID_Printf(PRINT_ALL, "Okay\n");
+		}
+		else
+		{
+			VID_Printf(PRINT_ALL, "Disabled\n");
+		}
+	}
+	else
+	{
+		VID_Printf(PRINT_ALL, "Failed\n");
+	}
+
+	// --------
+
+	/* Anisotropic */
+	VID_Printf(PRINT_ALL, " - Anisotropic: ");
 
 	if (strstr(gl_config.extensions_string, "GL_EXT_texture_filter_anisotropic"))
 	{
-		VID_Printf(PRINT_ALL, "...using GL_EXT_texture_filter_anisotropic\n");
 		gl_config.anisotropic = true;
 		glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &gl_config.max_anisotropy);
-		Cvar_SetValue("gl_anisotropic_avail", gl_config.max_anisotropy);
+
+		VID_Printf(PRINT_ALL, "%ux\n", (int)gl_config.max_anisotropy);
 	}
 	else
 	{
-		VID_Printf(PRINT_ALL, "...GL_EXT_texture_filter_anisotropic not found\n");
 		gl_config.anisotropic = false;
 		gl_config.max_anisotropy = 0.0;
-		Cvar_SetValue("gl_anisotropic_avail", 0.0);
+
+		VID_Printf(PRINT_ALL, "Failed\n");
 	}
 
+	// ----
+
+	/* Non power of two textures */
+	VID_Printf(PRINT_ALL, " - Non power of two textures: ");
+
 	if (strstr(gl_config.extensions_string, "GL_ARB_texture_non_power_of_two"))
 	{
-		VID_Printf(PRINT_ALL, "...using GL_ARB_texture_non_power_of_two\n");
-		gl_config.tex_npot = true;
-	}
-
-	gl_config.mtexcombine = false;
-
-	if (strstr(gl_config.extensions_string, "GL_ARB_texture_env_combine"))
-	{
-		if (gl_ext_mtexcombine->value)
-		{
-			VID_Printf(PRINT_ALL, "...using GL_ARB_texture_env_combine\n");
-			gl_config.mtexcombine = true;
-		}
-		else
-		{
-			VID_Printf(PRINT_ALL, "...ignoring GL_ARB_texture_env_combine\n");
-		}
+		gl_config.npottextures = true;
+		VID_Printf(PRINT_ALL, "Okay\n");
 	}
 	else
 	{
-		VID_Printf(PRINT_ALL, "...GL_ARB_texture_env_combine not found\n");
+		gl_config.npottextures = false;
+		VID_Printf(PRINT_ALL, "Failed\n");
 	}
 
-	if (!gl_config.mtexcombine)
-	{
-		if (strstr(gl_config.extensions_string, "GL_EXT_texture_env_combine"))
-		{
-			if (gl_ext_mtexcombine->value)
-			{
-				VID_Printf(PRINT_ALL, "...using GL_EXT_texture_env_combine\n");
-				gl_config.mtexcombine = true;
-			}
-			else
-			{
-				VID_Printf(PRINT_ALL, "...ignoring GL_EXT_texture_env_combine\n");
-			}
-		}
-		else
-		{
-			VID_Printf(PRINT_ALL, "...GL_EXT_texture_env_combine not found\n");
-		}
-	}
+	// ----
 
 	R_SetDefaultState();
 
@@ -1302,13 +1570,6 @@ R_Init(void *hinstance, void *hWnd)
 	R_InitParticleTexture();
 	Draw_InitLocal();
 
-	err = glGetError();
-
-	if (err != GL_NO_ERROR)
-	{
-		VID_Printf(PRINT_ALL, "glGetError() = 0x%x\n", err);
-	}
-
 	return true;
 }
 
@@ -1344,6 +1605,21 @@ R_BeginFrame(float camera_separation)
 		vid_fullscreen->modified = true;
 	}
 
+	// force a vid_restart if gl_stereo has been modified.
+	if ( gl_state.stereo_mode != gl_stereo->value ) {
+		// If we've gone from one mode to another with the same special buffer requirements there's no need to restart.
+		if ( GL_GetSpecialBufferModeForStereoMode( gl_state.stereo_mode ) == GL_GetSpecialBufferModeForStereoMode( gl_stereo->value )  ) {
+			gl_state.stereo_mode = gl_stereo->value;
+		}
+		else
+		{
+			VID_Printf(PRINT_ALL, "stereo supermode changed, restarting video!\n");
+			cvar_t	*ref;
+			ref = Cvar_Get("vid_fullscreen", "0", CVAR_ARCHIVE);
+			ref->modified = true;
+		}
+	}
+
 	if (vid_gamma->modified)
 	{
 		vid_gamma->modified = false;
@@ -1354,8 +1630,44 @@ R_BeginFrame(float camera_separation)
 		}
 	}
 
+	// Clamp overbrightbits
+	if (gl_overbrightbits->modified)
+	{
+		if (gl_overbrightbits->value > 2 && gl_overbrightbits->value < 4)
+		{
+			Cvar_Set("gl_overbrightbits", "2");
+		}
+		else if (gl_overbrightbits->value > 4)
+		{
+			Cvar_Set("gl_overbrightbits", "4");
+		}
+
+		gl_overbrightbits->modified = false;
+	}
+
 	/* go into 2D mode */
-	glViewport(0, 0, vid.width, vid.height);
+
+	int x, w, y, h;
+	qboolean drawing_left_eye = gl_state.camera_separation < 0;
+	qboolean stereo_split_tb = ((gl_state.stereo_mode == STEREO_SPLIT_VERTICAL) && gl_state.camera_separation);
+	qboolean stereo_split_lr = ((gl_state.stereo_mode == STEREO_SPLIT_HORIZONTAL) && gl_state.camera_separation);
+
+	x = 0;
+	w = vid.width;
+	y = 0;
+	h = vid.height;
+
+	if(stereo_split_lr) {
+		w =  w / 2;
+		x = drawing_left_eye ? 0 : w;
+	}
+
+	if(stereo_split_tb) {
+		h =  h / 2;
+		y = drawing_left_eye ? h : 0;
+	}
+
+	glViewport(x, y, w, h);
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
 	glOrtho(0, vid.width, vid.height, 0, -99999, 99999);
@@ -1372,7 +1684,7 @@ R_BeginFrame(float camera_separation)
 	{
 		gl_drawbuffer->modified = false;
 
-		if ((gl_state.camera_separation == 0) || !gl_state.stereo_enabled)
+		if ((gl_state.camera_separation == 0) || gl_state.stereo_mode != STEREO_MODE_OPENGL)
 		{
 			if (Q_stricmp(gl_drawbuffer->string, "GL_FRONT") == 0)
 			{
@@ -1455,7 +1767,11 @@ R_DrawBeam(entity_t *e)
 	vec3_t direction, normalized_direction;
 	vec3_t start_points[NUM_BEAM_SEGS], end_points[NUM_BEAM_SEGS];
 	vec3_t oldorigin, origin;
-
+ 
+	GLfloat vtx[3*NUM_BEAM_SEGS*4];
+	unsigned int index_vtx = 0;
+	unsigned int pointb;
+ 
 	oldorigin[0] = e->oldorigin[0];
 	oldorigin[1] = e->oldorigin[1];
 	oldorigin[2] = e->oldorigin[2];
@@ -1498,27 +1814,35 @@ R_DrawBeam(entity_t *e)
 
 	glColor4f(r, g, b, e->alpha);
 
-	glBegin(GL_TRIANGLE_STRIP);
-
-	for (i = 0; i < NUM_BEAM_SEGS; i++)
+	for ( i = 0; i < NUM_BEAM_SEGS; i++ )
 	{
-		glVertex3fv(start_points[i]);
-		glVertex3fv(end_points[i]);
-		glVertex3fv(start_points[(i + 1) % NUM_BEAM_SEGS]);
-		glVertex3fv(end_points[(i + 1) % NUM_BEAM_SEGS]);
+		vtx[index_vtx++] = start_points [ i ][ 0 ];
+		vtx[index_vtx++] = start_points [ i ][ 1 ];
+		vtx[index_vtx++] = start_points [ i ][ 2 ];
+
+		vtx[index_vtx++] = end_points [ i ][ 0 ];
+		vtx[index_vtx++] = end_points [ i ][ 1 ];
+		vtx[index_vtx++] = end_points [ i ][ 2 ];
+
+		pointb = ( i + 1 ) % NUM_BEAM_SEGS;
+		vtx[index_vtx++] = start_points [ pointb ][ 0 ];
+		vtx[index_vtx++] = start_points [ pointb ][ 1 ];
+		vtx[index_vtx++] = start_points [ pointb ][ 2 ];
+
+		vtx[index_vtx++] = end_points [ pointb ][ 0 ];
+		vtx[index_vtx++] = end_points [ pointb ][ 1 ];
+		vtx[index_vtx++] = end_points [ pointb ][ 2 ];
 	}
 
-	glEnd();
+	glEnableClientState( GL_VERTEX_ARRAY );
+
+	glVertexPointer( 3, GL_FLOAT, 0, vtx );
+	glDrawArrays( GL_TRIANGLE_STRIP, 0, NUM_BEAM_SEGS*4 );
+
+	glDisableClientState( GL_VERTEX_ARRAY );
 
 	glEnable(GL_TEXTURE_2D);
 	glDisable(GL_BLEND);
 	glDepthMask(GL_TRUE);
 }
 
-/*void
-R_GetRefAPI(void)
-{
-	Swap_Init();
-}*/
-
-
