@@ -68,19 +68,19 @@ R_TextureAnimation(mtexinfo_t *tex)
 void
 R_DrawGLPoly(glpoly_t *p)
 {
-	int i;
 	float *v;
 
-	glBegin(GL_POLYGON);
 	v = p->verts[0];
 
-	for (i = 0; i < p->numverts; i++, v += VERTEXSIZE)
-	{
-		glTexCoord2f(v[3], v[4]);
-		glVertex3fv(v);
-	}
+    glEnableClientState( GL_VERTEX_ARRAY );
+    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
 
-	glEnd();
+    glVertexPointer( 3, GL_FLOAT, VERTEXSIZE*sizeof(GLfloat), v );
+    glTexCoordPointer( 2, GL_FLOAT, VERTEXSIZE*sizeof(GLfloat), v+3 );
+    glDrawArrays( GL_TRIANGLE_FAN, 0, p->numverts );
+
+    glDisableClientState( GL_VERTEX_ARRAY );
+    glDisableClientState( GL_TEXTURE_COORD_ARRAY );
 }
 
 void
@@ -100,16 +100,27 @@ R_DrawGLFlowingPoly(msurface_t *fa)
 		scroll = -64.0;
 	}
 
-	glBegin(GL_POLYGON);
-	v = p->verts[0];
+    GLfloat tex[2*p->numverts];
+    unsigned int index_tex = 0;
 
-	for (i = 0; i < p->numverts; i++, v += VERTEXSIZE)
-	{
-		glTexCoord2f((v[3] + scroll), v[4]);
-		glVertex3fv(v);
-	}
+    v = p->verts [ 0 ];
 
-	glEnd();
+	for ( i = 0; i < p->numverts; i++, v += VERTEXSIZE )
+    {
+        tex[index_tex++] = v [ 3 ] + scroll;
+        tex[index_tex++] = v [ 4 ];
+    }
+    v = p->verts [ 0 ];
+
+    glEnableClientState( GL_VERTEX_ARRAY );
+    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+
+    glVertexPointer( 3, GL_FLOAT, VERTEXSIZE*sizeof(GLfloat), v );
+    glTexCoordPointer( 2, GL_FLOAT, 0, tex );
+    glDrawArrays( GL_TRIANGLE_FAN, 0, p->numverts );
+
+    glDisableClientState( GL_VERTEX_ARRAY );
+    glDisableClientState( GL_TEXTURE_COORD_ARRAY );
 }
 
 void
@@ -141,12 +152,23 @@ R_DrawTriangleOutlines(void)
 			{
 				for (j = 2; j < p->numverts; j++)
 				{
-					glBegin(GL_LINE_STRIP);
-					glVertex3fv(p->verts[0]);
-					glVertex3fv(p->verts[j - 1]);
-					glVertex3fv(p->verts[j]);
-					glVertex3fv(p->verts[0]);
-					glEnd();
+                    GLfloat vtx[12];
+                    unsigned int k;
+
+                    for (k=0; k<3; k++)
+                    {
+                       vtx[0+k] = p->verts [ 0 ][ k ];
+                        vtx[3+k] = p->verts [ j - 1 ][ k ];
+                        vtx[6+k] = p->verts [ j ][ k ];
+                        vtx[9+k] = p->verts [ 0 ][ k ];
+                    }
+
+                    glEnableClientState( GL_VERTEX_ARRAY );
+
+                    glVertexPointer( 3, GL_FLOAT, 0, vtx );
+                    glDrawArrays( GL_LINE_STRIP, 0, 4 );
+
+                    glDisableClientState( GL_VERTEX_ARRAY );
 				}
 			}
 		}
@@ -164,7 +186,6 @@ R_DrawGLPolyChain(glpoly_t *p, float soffset, float toffset)
 		for ( ; p != 0; p = p->chain)
 		{
 			float *v;
-			int j;
 
 			v = p->verts[0];
 
@@ -174,15 +195,15 @@ R_DrawGLPolyChain(glpoly_t *p, float soffset, float toffset)
 				return;
 			}
 
-			glBegin(GL_POLYGON);
+            glEnableClientState( GL_VERTEX_ARRAY );
+            glEnableClientState( GL_TEXTURE_COORD_ARRAY );
 
-			for (j = 0; j < p->numverts; j++, v += VERTEXSIZE)
-			{
-				glTexCoord2f(v[5], v[6]);
-				glVertex3fv(v);
-			}
+            glVertexPointer( 3, GL_FLOAT, VERTEXSIZE*sizeof(GLfloat), v );
+            glTexCoordPointer( 2, GL_FLOAT, VERTEXSIZE*sizeof(GLfloat), v+5 );
+            glDrawArrays( GL_TRIANGLE_FAN, 0, p->numverts );
 
-			glEnd();
+            glDisableClientState( GL_VERTEX_ARRAY );
+            glDisableClientState( GL_TEXTURE_COORD_ARRAY );
 		}
 	}
 	else
@@ -192,16 +213,28 @@ R_DrawGLPolyChain(glpoly_t *p, float soffset, float toffset)
 			float *v;
 			int j;
 
-			glBegin(GL_POLYGON);
 			v = p->verts[0];
 
-			for (j = 0; j < p->numverts; j++, v += VERTEXSIZE)
+            GLfloat tex[2*p->numverts];
+            unsigned int index_tex = 0;
+
+			for ( j = 0; j < p->numverts; j++, v += VERTEXSIZE )
 			{
-				glTexCoord2f(v[5] - soffset, v[6] - toffset);
-				glVertex3fv(v);
+			    tex[index_tex++] = v [ 5 ] - soffset;
+			    tex[index_tex++] = v [ 6 ] - toffset;
 			}
 
-			glEnd();
+			v = p->verts [ 0 ];
+
+            glEnableClientState( GL_VERTEX_ARRAY );
+            glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+
+            glVertexPointer( 3, GL_FLOAT, VERTEXSIZE*sizeof(GLfloat), v );
+            glTexCoordPointer( 2, GL_FLOAT, 0, tex );
+            glDrawArrays( GL_TRIANGLE_FAN, 0, p->numverts );
+
+            glDisableClientState( GL_VERTEX_ARRAY );
+            glDisableClientState( GL_TEXTURE_COORD_ARRAY );
 		}
 	}
 }
@@ -269,6 +302,14 @@ R_BlendLightmaps(void)
 			{
 				if (surf->polys)
 				{
+					// Apply overbright bits to the static lightmaps
+					if (gl_overbrightbits->value)
+					{
+						R_TexEnv(GL_COMBINE_EXT);
+						R_SelectTexture(GL_TEXTURE1);
+						glTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, gl_overbrightbits->value);
+					}
+
 					R_DrawGLPolyChain(surf->polys, 0, 0);
 				}
 			}
@@ -321,6 +362,14 @@ R_BlendLightmaps(void)
 				{
 					if (drawsurf->polys)
 					{
+						// Apply overbright bits to the dynamic lightmaps
+						if (gl_overbrightbits->value)
+						{
+							R_TexEnv(GL_COMBINE_EXT);
+							R_SelectTexture(GL_TEXTURE1);
+							glTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, gl_overbrightbits->value);
+						}
+
 						R_DrawGLPolyChain(drawsurf->polys,
 								(drawsurf->light_s - drawsurf->dlight_s) * (1.0 / 128.0),
 								(drawsurf->light_t - drawsurf->dlight_t) * (1.0 / 128.0));
@@ -358,6 +407,14 @@ R_BlendLightmaps(void)
 		{
 			if (surf->polys)
 			{
+				// Apply overbright bits to the remainder lightmaps
+				if (gl_overbrightbits->value)
+				{
+					R_TexEnv(GL_COMBINE_EXT);
+					R_SelectTexture(GL_TEXTURE1);
+					glTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, gl_overbrightbits->value);
+				}
+
 				R_DrawGLPolyChain(surf->polys,
 						(surf->light_s - surf->dlight_s) * (1.0 / 128.0),
 						(surf->light_t - surf->dlight_t) * (1.0 / 128.0));
@@ -386,10 +443,32 @@ R_RenderBrushPoly(msurface_t *fa)
 	{
 		R_Bind(image->texnum);
 
-		/* warp texture, no lightmaps */
-		R_TexEnv(GL_MODULATE);
-		glColor4f(gl_state.inverse_intensity, gl_state.inverse_intensity,
-				gl_state.inverse_intensity, 1.0F);
+		/* This is a hack ontop of a hack. Warping surfaces like those generated
+		   by R_EmitWaterPolys() don't have a lightmap. Original Quake II therefor
+		   negated the global intensity on those surfaces, because otherwise they
+		   would show up much too bright. When we implemented overbright bits this
+		   hack modified the global GL state in an incompatible way. So implement
+		   a new hack, based on overbright bits... Depending on the value set to
+		   gl_overbrightbits the result is different:
+
+		    0: Old behaviour.
+		    1: No overbright bits on the global scene but correct lightning on
+		       warping surfaces.
+		    2: Overbright bits on the global scene but not on warping surfaces.
+		        They oversaturate otherwise. */
+		if (gl_overbrightbits->value)
+		{
+			R_TexEnv(GL_COMBINE_EXT);
+			R_SelectTexture(GL_TEXTURE1);
+			glTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, 1);
+		}
+		else
+		{
+			R_TexEnv(GL_MODULATE);
+			glColor4f(gl_state.inverse_intensity, gl_state.inverse_intensity,
+					  gl_state.inverse_intensity, 1.0f);
+		}
+
 		R_EmitWaterPolys(fa);
 		R_TexEnv(GL_REPLACE);
 
@@ -542,7 +621,7 @@ R_DrawTextureChains(void)
 
 	c_visible_textures = 0;
 
-	if (!qglActiveTextureARB)
+	if (!gl_config.multitexture)
 	{
 		for (i = 0, image = gltextures; i < numgltextures; i++, image++)
 		{
@@ -624,34 +703,45 @@ R_DrawTextureChains(void)
 	R_TexEnv(GL_REPLACE);
 }
 
-static void
+void
 R_RenderLightmappedPoly(msurface_t *surf)
 {
-	int i, nv = surf->polys->numverts;
+	int i;
 	int map;
+	int nv;
+	int smax;
+	int tmax;
+	float scroll;
 	float *v;
-	image_t *image = R_TextureAnimation(surf->texinfo);
-	qboolean is_dynamic = false;
-	unsigned lmtex = surf->lightmaptexturenum;
 	glpoly_t *p;
+	image_t *image;
+	qboolean is_dynamic;
+	unsigned lmtex;
+	unsigned temp[128 * 128];
 
+	image = R_TextureAnimation(surf->texinfo);
+	is_dynamic = false;
+	lmtex = surf->lightmaptexturenum;
+	nv = surf->polys->numverts;
+
+	// Any dynamic lights on this surface?
 	for (map = 0; map < MAXLIGHTMAPS && surf->styles[map] != 255; map++)
 	{
-		if (r_newrefdef.lightstyles[surf->styles[map]].white !=
-			surf->cached_light[map])
+		if (r_newrefdef.lightstyles[surf->styles[map]].white != surf->cached_light[map])
 		{
-			goto dynamic;
+			if (!(surf->texinfo->flags & (SURF_SKY | SURF_TRANS33 | SURF_TRANS66 | SURF_WARP)))
+			{
+				is_dynamic = true;
+			}
 		}
 	}
 
+	// Normal dynamic lights
 	if (surf->dlightframe == r_framecount)
 	{
-	dynamic:
-
 		if (gl_dynamic->value)
 		{
-			if (!(surf->texinfo->flags &
-				  (SURF_SKY | SURF_TRANS33 | SURF_TRANS66 | SURF_WARP)))
+			if (!(surf->texinfo->flags & (SURF_SKY | SURF_TRANS33 | SURF_TRANS66 | SURF_WARP)))
 			{
 				is_dynamic = true;
 			}
@@ -660,140 +750,97 @@ R_RenderLightmappedPoly(msurface_t *surf)
 
 	if (is_dynamic)
 	{
-		unsigned temp[128 * 128];
-		int smax, tmax;
-
-		if (((surf->styles[map] >= 32) ||
-			 (surf->styles[map] == 0)) &&
-				(surf->dlightframe != r_framecount))
+		// Dynamic lights on a surface
+		if (((surf->styles[map] >= 32) || (surf->styles[map] == 0)) && (surf->dlightframe != r_framecount))
 		{
 			smax = (surf->extents[0] >> 4) + 1;
 			tmax = (surf->extents[1] >> 4) + 1;
 
-			R_BuildLightMap(surf, (void *)temp, smax * 4);
+			R_BuildLightMap(surf, (void *) temp, smax * 4);
 			R_SetCacheState(surf);
-
 			R_MBind(GL_TEXTURE1_ARB, gl_state.lightmap_textures + surf->lightmaptexturenum);
 
 			lmtex = surf->lightmaptexturenum;
 
-			glTexSubImage2D(GL_TEXTURE_2D, 0, surf->light_s, surf->light_t,
-					smax, tmax, GL_LIGHTMAP_FORMAT, GL_UNSIGNED_BYTE, temp);
+			glTexSubImage2D(GL_TEXTURE_2D, 0, surf->light_s, surf->light_t, smax,
+							tmax, GL_LIGHTMAP_FORMAT, GL_UNSIGNED_BYTE, temp);
 		}
-		else
+		else // Normal dynamic lights
 		{
 			smax = (surf->extents[0] >> 4) + 1;
 			tmax = (surf->extents[1] >> 4) + 1;
 
-			R_BuildLightMap(surf, (void *)temp, smax * 4);
-
+			R_BuildLightMap(surf, (void *) temp, smax * 4);
 			R_MBind(GL_TEXTURE1_ARB, gl_state.lightmap_textures + 0);
 
 			lmtex = 0;
 
-			glTexSubImage2D(GL_TEXTURE_2D, 0, surf->light_s, surf->light_t,
-					smax, tmax, GL_LIGHTMAP_FORMAT, GL_UNSIGNED_BYTE, temp);
+			glTexSubImage2D(GL_TEXTURE_2D, 0, surf->light_s, surf->light_t, smax,
+							tmax, GL_LIGHTMAP_FORMAT, GL_UNSIGNED_BYTE, temp);
 		}
 
 		c_brush_polys++;
 
 		R_MBind(GL_TEXTURE0_ARB, image->texnum);
 		R_MBind(GL_TEXTURE1_ARB, gl_state.lightmap_textures + lmtex);
-
-		if (surf->texinfo->flags & SURF_FLOWING)
-		{
-			float scroll;
-
-			scroll = -64 *
-					 ((r_newrefdef.time / 40.0) - (int)(r_newrefdef.time / 40.0));
-
-			if (scroll == 0.0)
-			{
-				scroll = -64.0;
-			}
-
-			for (p = surf->polys; p; p = p->chain)
-			{
-				v = p->verts[0];
-				glBegin(GL_POLYGON);
-
-				for (i = 0; i < nv; i++, v += VERTEXSIZE)
-				{
-					qglMultiTexCoord2fARB(GL_TEXTURE0_ARB, (v[3] + scroll), v[4]);
-					qglMultiTexCoord2fARB(GL_TEXTURE1_ARB, v[5], v[6]);
-					glVertex3fv(v);
-				}
-
-				glEnd();
-			}
-		}
-		else
-		{
-			for (p = surf->polys; p; p = p->chain)
-			{
-				v = p->verts[0];
-				glBegin(GL_POLYGON);
-
-				for (i = 0; i < nv; i++, v += VERTEXSIZE)
-				{
-					qglMultiTexCoord2fARB(GL_TEXTURE0_ARB, v[3], v[4]);
-					qglMultiTexCoord2fARB(GL_TEXTURE1_ARB, v[5], v[6]);
-					glVertex3fv(v);
-				}
-
-				glEnd();
-			}
-		}
 	}
-	else
+	else // No dynamic lights
 	{
 		c_brush_polys++;
 
 		R_MBind(GL_TEXTURE0_ARB, image->texnum);
 		R_MBind(GL_TEXTURE1_ARB, gl_state.lightmap_textures + lmtex);
+	}
 
-		if (surf->texinfo->flags & SURF_FLOWING)
+	if (surf->texinfo->flags & SURF_FLOWING)
+	{
+		scroll = -64 * ((r_newrefdef.time / 40.0) - (int) (r_newrefdef.time / 40.0));
+
+		if (scroll == 0.0)
 		{
-			float scroll;
-
-			scroll = -64 * ((r_newrefdef.time / 40.0) - (int)(r_newrefdef.time / 40.0));
-
-			if (scroll == 0.0)
-			{
-				scroll = -64.0;
-			}
-
-			for (p = surf->polys; p; p = p->chain)
-			{
-				v = p->verts[0];
-				glBegin(GL_POLYGON);
-
-				for (i = 0; i < nv; i++, v += VERTEXSIZE)
-				{
-					qglMultiTexCoord2fARB(GL_TEXTURE0_ARB, (v[3] + scroll), v[4]);
-					qglMultiTexCoord2fARB(GL_TEXTURE1_ARB, v[5], v[6]);
-					glVertex3fv(v);
-				}
-
-				glEnd();
-			}
+			scroll = -64.0;
 		}
-		else
+
+		for (p = surf->polys; p; p = p->chain)
 		{
-			for (p = surf->polys; p; p = p->chain)
+			v = p->verts[0];
+			glBegin(GL_POLYGON);
+
+			for (i = 0; i < nv; i++, v += VERTEXSIZE)
 			{
-				v = p->verts[0];
-				glBegin(GL_POLYGON);
-
-				for (i = 0; i < nv; i++, v += VERTEXSIZE)
-				{
-					qglMultiTexCoord2fARB(GL_TEXTURE0_ARB, v[3], v[4]);
-					qglMultiTexCoord2fARB(GL_TEXTURE1_ARB, v[5], v[6]);
-					glVertex3fv(v);
-				}
-
-				glEnd();
+				qglMultiTexCoord2fARB(GL_TEXTURE0, (v[3] + scroll), v[4]);
+				qglMultiTexCoord2fvARB(GL_TEXTURE1, &v[5]);
+				glVertex3fv(v);
 			}
+
+			glEnd();
+		}
+	}
+	else
+	{
+		for (p = surf->polys; p; p = p->chain)
+		{
+			v = p->verts[0];
+
+			// Polygon
+			glEnableClientState(GL_VERTEX_ARRAY);
+			glVertexPointer(3, GL_FLOAT, VERTEXSIZE * sizeof(GLfloat), v);
+
+			// Texture
+			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+			qglClientActiveTextureARB(GL_TEXTURE0_ARB);
+			glTexCoordPointer(2, GL_FLOAT, VERTEXSIZE * sizeof(GLfloat), v + 3);
+
+			// Lightmap
+			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+			qglClientActiveTextureARB(GL_TEXTURE1_ARB);
+			glTexCoordPointer(2, GL_FLOAT, VERTEXSIZE * sizeof(GLfloat), v + 5);
+
+			// Draw the crap
+			glDrawArrays(GL_TRIANGLE_FAN, 0, p->numverts);
+
+			glDisableClientState(GL_VERTEX_ARRAY);
+			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
 		}
 	}
 }
@@ -814,8 +861,7 @@ R_DrawInlineBModel(void)
 
 		for (k = 0; k < r_newrefdef.num_dlights; k++, lt++)
 		{
-			R_MarkLights(lt, 1 << k,
-					currentmodel->nodes + currentmodel->firstnode);
+			R_MarkLights(lt, 1 << k, currentmodel->nodes + currentmodel->firstnode);
 		}
 	}
 
@@ -846,7 +892,7 @@ R_DrawInlineBModel(void)
 				psurf->texturechain = r_alpha_surfaces;
 				r_alpha_surfaces = psurf;
 			}
-			else if (qglMultiTexCoord2fARB && !(psurf->flags & SURF_DRAWTURB))
+			else if (gl_config.multitexture && !(psurf->flags & SURF_DRAWTURB))
 			{
 				R_RenderLightmappedPoly(psurf);
 			}
@@ -861,7 +907,7 @@ R_DrawInlineBModel(void)
 
 	if (!(currententity->flags & RF_TRANSLUCENT))
 	{
-		if (!qglMultiTexCoord2fARB)
+		if (!gl_config.multitexture)
 		{
 			R_BlendLightmaps();
 		}
@@ -916,7 +962,7 @@ R_DrawBrushModel(entity_t *e)
 		glEnable(GL_POLYGON_OFFSET_FILL);
 	}
 
-	glColor3f(1, 1, 1);
+	glColor4f(1, 1, 1, 1);
 	memset(gl_lms.lightmap_surfaces, 0, sizeof(gl_lms.lightmap_surfaces));
 
 	VectorSubtract(r_newrefdef.vieworg, e->origin, modelorg);
@@ -1121,7 +1167,7 @@ R_RecursiveWorldNode(mnode_t *node)
 		}
 		else
 		{
-			if (qglMultiTexCoord2fARB && !(surf->flags & SURF_DRAWTURB))
+			if (gl_config.multitexture && !(surf->flags & SURF_DRAWTURB))
 			{
 				R_RenderLightmappedPoly(surf);
 			}
@@ -1165,11 +1211,11 @@ R_DrawWorld(void)
 
 	gl_state.currenttextures[0] = gl_state.currenttextures[1] = -1;
 
-	glColor3f(1, 1, 1);
+	glColor4f(1, 1, 1, 1);
 	memset(gl_lms.lightmap_surfaces, 0, sizeof(gl_lms.lightmap_surfaces));
 	R_ClearSkyBox();
 
-	if (qglMultiTexCoord2fARB)
+	if (gl_config.multitexture)
 	{
 		R_EnableMultitexture(true);
 
@@ -1178,7 +1224,7 @@ R_DrawWorld(void)
 		if (!gl_config.mtexcombine)
 		{
 			R_TexEnv(GL_REPLACE);
-			R_SelectTexture(GL_TEXTURE1);
+			R_SelectTexture(GL_TEXTURE1_ARB);
 
 			if (gl_lightmap->value)
 			{
@@ -1196,7 +1242,7 @@ R_DrawWorld(void)
 			glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_TEXTURE);
 			glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_EXT, GL_REPLACE);
 			glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_EXT, GL_TEXTURE);
-			R_SelectTexture(GL_TEXTURE1);
+			R_SelectTexture(GL_TEXTURE1_ARB);
 			R_TexEnv(GL_COMBINE_EXT);
 
 			if (gl_lightmap->value)
