@@ -33,34 +33,19 @@
 #include "crc.h"
 
 /* Should have 4 characters. */
-#define YQ2VERSION "5.32"
-
+#define YQ2VERSION "6.00"
 #define BASEDIRNAME "baseq2"
 
-#if defined __linux__
- #define BUILDSTRING "Linux"
-#elif defined __FreeBSD__
- #define BUILDSTRING "FreeBSD"
-#elif defined __OpenBSD__
- #define BUILDSTRING "OpenBSD"
-#elif defined _WIN32
- #define BUILDSTRING "Windows"
-#elif defined __APPLE__
- #define BUILDSTRING "MacOS X"
-#else
- #define BUILDSTRING "Unknown"
+#ifndef YQ2OSTYPE
+#error YQ2OSTYPE should be defined by the build system
 #endif
 
-#ifdef __i386__
- #define CPUSTRING "i386"
-#elif defined __x86_64__
- #define CPUSTRING "amd64"
-#elif defined __sparc__
- #define CPUSTRING "sparc64"
-#elif defined __ia64__
- #define CPUSTRING "ia64"
-#else
- #define CPUSTRING "Unknown"
+#ifndef YQ2ARCH
+#error YQ2ARCH should be defined by the build system
+#endif
+
+#ifndef BUILD_DATE
+#define BUILD_DATE __DATE__
 #endif
 
 #ifdef _WIN32
@@ -79,7 +64,6 @@
  #define LIBGL "libGL.so.1"
 #endif
 
-
 /* ================================================================== */
 
 typedef struct sizebuf_s
@@ -767,7 +751,7 @@ extern vec3_t bytedirs[NUMVERTEXNORMALS];
 /* this is in the client code, but can be used for debugging from server */
 void SCR_DebugGraph(float value, int color);
 
-/* NON-PORTABLE SYSTEM SERVICES */
+/* NON-PORTABLE OSTYPE SERVICES */
 
 void Sys_Init(void);
 void Sys_UnloadGame(void);
@@ -780,6 +764,7 @@ void Sys_Error(char *error, ...);
 void Sys_Quit(void);
 char *Sys_GetHomeDir(void);
 const char *Sys_GetBinaryDir(void);
+void Sys_Sleep(int msec);
 
 void Sys_FreeLibrary(void *handle);
 void *Sys_LoadLibrary(const char *path, const char *sym, void **handle);
